/*******************************************************************************
 *
 * libgui Version 0.0.1
 *
 * libgui is a simple framework of GUI classes for X11/Motif widgets written in C++.
 *
 * The full text of the GNU General Public License can be found in the doc 
 * directory.
 *
 * Copyright (C) 2002, 2003 Stefan Hoefer
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * You can contact Stefan Hoefer via email at stefan@hoefer.ch
 *
 ******************************************************************************/

#include <Xm/Xm.h>
#include <Xm/List.h>

#include "commonlistbox.h"
#include <basic/iterator.h>

CCommonListBox::~CCommonListBox()
{
    CSimpleIterator<CIdString *> StringIt;
    
    StringIt << &m_Strings;
    
    while ( CIdString *pString = StringIt++ )
    {
	delete pString;
    }
}

void CCommonListBox::operator+=(const CIdString &String)
{
    CIdString *NewString = new CIdString;
	
    *NewString = String;
	
    int nInsertPosition = 0;

    if (Style() & LB_STYLE_SORTED)
    {
	bool bGefunden = (m_Strings.card()==0);
	
	int nStart = 0;
	int nEnd = m_Strings.card()-1;

	while (!bGefunden)
	{
	    if (nStart <= nEnd)
	    {
		nInsertPosition = nStart + (nEnd - nStart)/2;

		CStrings *pString = m_Strings[nInsertPosition];

		if (*pString == *NewString)
		{
		    bGefunden = true;
		}
		else if (*pString < *NewString)
		{
		    nStart = nInsertPosition+1;
		    nInsertPosition = nStart;
		}
		else if (*pString > *NewString)
		{
		    nEnd = nInsertPosition-1;
		    nInsertPosition = nStart;
		}
	    }
	    else
	    {
		bGefunden = true;
	    }
	}

	m_Strings.InsertAt(nInsertPosition, NewString);
    }
    else
    {
	m_Strings += NewString;
	nInsertPosition = -1;
    }

    char *strBuffer = new char[String.Length()+1];
    strcpy(strBuffer,(const char *)String);
	
    XmString string = XmStringCreate(strBuffer,XmFONTLIST_DEFAULT_TAG);
    delete strBuffer;
	
    XmListAddItem(m_List,string,nInsertPosition+1);
    XmStringFree(string);

    if (nInsertPosition > 0 && m_nCurrentSelection >= nInsertPosition)
    {
	Select(m_nCurrentSelection+1);
    }
}

void CCommonListBox::operator-=(const CIdString &strTest)
{
    for (int nIndex = 0; nIndex < Strings().card(); nIndex++)
    {
	if (strTest == *Strings()[nIndex])
	{
	    RemoveAt(nIndex);
	}
    }
}

void CCommonListBox::RemoveAt(int nIndex)
{
    if (nIndex >= 0 && nIndex < Strings().card())
    {
	m_Strings.RemoveAt(nIndex);

	XmListDeletePos(m_List, nIndex+1);
    }
}

CIdString *CCommonListBox::GetCurrentSelection()
{
    if ( m_nCurrentSelection >= 0 )
    {
	return m_Strings[m_nCurrentSelection];
    }
    else
    {
	return NULL;
    }
}

void CCommonListBox::ResetContents()
{
    CSimpleIterator<CIdString *> StringIt;
	
    StringIt << &m_Strings;
	
    while ( CIdString *pString = StringIt++ )
    {
	delete pString;
    }

    m_Strings.reset();
	
    XmListDeleteAllItems(m_List);	
}

void CCommonListBox::SetCurrentSelection(CIdString &string)
{
    SetCurrentSelection((const char *)string);
}

void CCommonListBox::SetCurrentSelection(const char *string)
{
    if ( m_nCurrentSelection >= 0 ){
	*(m_Strings[m_nCurrentSelection]) = string;
		
	char *strBuffer = new char[strlen(string)+1];
	strcpy(strBuffer,string);
		
	XmString strText = XmStringCreate(strBuffer,XmFONTLIST_DEFAULT_TAG);
	delete strBuffer;
		
	XmListReplaceItemsPos(m_List,&strText,1,m_nCurrentSelection+1);

	Arg args[5];
	Cardinal argcount = 0;
	XtSetArg(args[argcount],XmNselectedItem,strText); argcount++;
		
	XtSetValues(Container(),args,argcount);
		
	//XmStringFree(strText);
    }
}

void CCommonListBox::Select(int nSelection)
{
    if (nSelection >= 0)
    {
	if (nSelection < card())
	{
	    m_nCurrentSelection = nSelection;
	    /*Arg args[5];
	      Cardinal argcount=0;
			
	      XtSetArg(args[argcount],XmNselectedPosition,nSelection); argcount++;
	      XtSetValues(Container(),args,argcount);*/
			
	    // Does not work! (circles) XmListSelectPos(m_List,nSelection+1,TRUE);
	    m_bInternalSelect = true;
	    //XmListSelectPos(m_List,nSelection+1,FALSE);
	    XmListSelectPos(m_List,nSelection+1,TRUE);
	    m_bInternalSelect = false;
	    //XmListSelectPos(m_List,nSelection,FALSE);
	}
    }
    else
    {
	m_nCurrentSelection = nSelection;
		
	XmListDeselectAllItems(m_List);
    }
}

/*void CCommonListBox::SelectionChanged()
{

}*/

void CCommonListBox::Select(IDENT lSelection)
{
    CIdString *pString;
	
    if ( lSelection > 0 )
    {
	for ( int nIndex = 0; nIndex < card(); nIndex++ )
	{
	    if ( m_Strings[nIndex]->Id() == lSelection )
	    {
		Select(nIndex);
		return;
	    }
	}
    }
    else
    {
	Select((int)-1);
    }
}

CCommonListBox::CCommonListBox(Widget wdgParent, unsigned long lStyle, int nId)
    : CGuiElement(wdgParent, lStyle, nId)
{
    m_nCurrentSelection = -1;
    m_bInternalSelect = false;

    //SetWorkaroundXtDestroyWidget();
}

void CCommonListBox::InternalCreate(unsigned long lStyle)
{
    InternalListCreate(lStyle);
	
    m_List = InternalList();

    Arg args[5];
    Cardinal argcount = 0;
	
    XtSetArg(args[argcount], XmNvisibleItemCount, 10); argcount++;

    XtSetValues(m_List, args, argcount);
}





