/*******************************************************************************
 *
 * libgui Version 0.0.1
 *
 * libgui is a simple framework of GUI classes for X11/Motif widgets written in C++.
 *
 * The full text of the GNU General Public License can be found in the doc 
 * directory.
 *
 * Copyright (C) 2002, 2003 Stefan Hoefer
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * You can contact Stefan Hoefer via email at stefan@hoefer.ch
 *
 ******************************************************************************/

#ifndef COMMONLISTBOX_H

#define COMMONLISTBOX_H

#include "guielement.h"
#include <basic/simplelist.h>

#define LB_STYLE_SORTED 1L
#define LB_STYLE_CONSTANTWIDTH 2L
#define LB_STYLE_MULTISELECT   4L

class CCommonListBox : public CGuiElement
{
 private:
    int m_nCurrentSelection;
    Widget m_List;
    CSimpleList<CIdString *> m_Strings;
    bool m_bInternalSelect;
 protected:
    virtual void SelectionChanged() = NULL;
    virtual Widget InternalList() = NULL;
    virtual void InternalCreate(unsigned long lStyle);
    virtual void InternalListCreate(unsigned long lStyle)=NULL;
		
    inline Widget List() { return m_List; }
		
    CCommonListBox(Widget wdgParent, unsigned long lStyle = 0L, int nId=0);
    ~CCommonListBox();
		
    inline int CurrentSelection() { return m_nCurrentSelection;}
    void SetCurrentSelection(int nCurrentSelection) { m_nCurrentSelection = nCurrentSelection; }

    inline bool InternalSelect() const { return m_bInternalSelect; }
 public:
    void operator+=(const CIdString &String);
    void operator-=(const CIdString &String);
    CIdString *GetCurrentSelection();
    void ResetContents();
    void SetCurrentSelection(CIdString &string);
    void SetCurrentSelection(const char *string);
    virtual void Select(int nSelection);
    void Select(IDENT lSelection);
    void RemoveAt(int nIndex);

    inline int card() { return m_Strings.card(); }
    inline const CSimpleList<CIdString *> &Strings() { return m_Strings; }
};

#endif
