/*******************************************************************************
 *
 * libgui Version 0.0.1
 *
 * libgui is a simple framework of GUI classes for X11/Motif widgets written in C++.
 *
 * The full text of the GNU General Public License can be found in the doc 
 * directory.
 *
 * Copyright (C) 2002, 2003 Stefan Hoefer
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * You can contact Stefan Hoefer via email at stefan@hoefer.ch
 *
 ******************************************************************************/

#include "dialog.h"

#include <Xm/Xm.h>
#include <Xm/PushB.h>
#include <Xm/Separator.h>
#include <Xm/Form.h>

#include <basic/globalstring.h>
#include "xapplication.h"

//#define DEFAULT_WIDTH 80
//#define DEFAULT_HEIGHT 30

Widget CDialog::Container() const
{
    return m_wdgShell;
}

CDialog::CDialog(long lStyle)
//    : CGuiElement(ApplicationShell(wdgParent), lStyle)
    : CGuiElement(xapplication::get_main_widget(), lStyle)
{
    m_wdgSeparator = NULL;
    m_OkButton = NULL;
    m_CancelButton = NULL;
    m_nReturnValue = 0;    
}

void CDialog::InternalCreate(unsigned long lStyle)
{
    Widget wdgForm = NULL;
    Widget wdgButton = NULL;
    Arg args[20];
    Cardinal argcount = 0;
	
    if ((Style() & DLG_STYLE_CANCEL)
	|| (Style() & DLG_STYLE_OK)
	|| (Style() & DLG_STYLE_SEPARATOR))
    {
	XtSetArg(args[argcount],XmNrightAttachment,XmATTACH_FORM); argcount++;
	XtSetArg(args[argcount],XmNbottomAttachment,XmATTACH_FORM); argcount++;
	XtSetArg(args[argcount],XmNtopAttachment,XmATTACH_NONE); argcount++;
	XtSetArg(args[argcount],XmNleftAttachment,XmATTACH_NONE); argcount++;
	
	wdgForm = XmCreateForm(Frame(),"buttonform",args,argcount);
	XtManageChild(wdgForm);
    }

    if (Style()&DLG_STYLE_CANCEL)
    {
	argcount = 0;

	XtSetArg(args[argcount],XmNleftAttachment,XmATTACH_NONE); argcount++;
	XtSetArg(args[argcount],XmNrightAttachment,XmATTACH_FORM); argcount++;
	XtSetArg(args[argcount],XmNbottomAttachment,XmATTACH_FORM); argcount++;
	XtSetArg(args[argcount],XmNtopAttachment,XmATTACH_FORM); argcount++;
		
	m_CancelButton = new CDialogButton(this, wdgForm,IDCANCEL);
	m_CancelButton->Create();
	wdgButton = m_CancelButton->Container();
	XtSetValues(m_CancelButton->Container(),args,argcount);
	m_CancelButton->SetCaption(LANGUAGE_STRING(IDS_CANCEL));
    }

    if (Style() & DLG_STYLE_OK)
    {
	argcount = 0;
	
	if ( wdgButton != NULL )
	{
	    XtSetArg(args[argcount],XmNrightAttachment,XmATTACH_WIDGET); argcount++;
	    XtSetArg(args[argcount],XmNrightWidget, wdgButton); argcount++;
	}
	else
	{
	    XtSetArg(args[argcount],XmNrightAttachment,XmATTACH_FORM); argcount++;
	}
	XtSetArg(args[argcount],XmNbottomAttachment,XmATTACH_FORM); argcount++;
	XtSetArg(args[argcount],XmNtopAttachment,XmATTACH_FORM); argcount++;
	XtSetArg(args[argcount],XmNleftAttachment,XmATTACH_FORM); argcount++;
		
	m_OkButton = new CDialogButton(this, wdgForm,IDOK);
	m_OkButton->Create();
	wdgButton = m_OkButton->Container();
	XtSetValues(m_OkButton->Container(),args,argcount);
	m_OkButton->SetCaption(LANGUAGE_STRING(IDS_OK));
    }

    if (Style() & DLG_STYLE_SEPARATOR)
    {
	argcount = 0;
	
	if ( wdgForm != NULL )
	{
	    XtSetArg(args[argcount],XmNbottomAttachment,XmATTACH_WIDGET); argcount++;
	    XtSetArg(args[argcount],XmNbottomWidget,wdgForm); argcount++;
	}
	else
	{
	    XtSetArg(args[argcount],XmNbottomAttachment,XmATTACH_FORM); argcount++;
	}
	XtSetArg(args[argcount],XmNleftAttachment,XmATTACH_FORM); argcount++;
	XtSetArg(args[argcount],XmNrightAttachment,XmATTACH_FORM); argcount++;
	XtSetArg(args[argcount],XmNtopAttachment, XmATTACH_NONE); argcount++;

	m_wdgSeparator = XmCreateSeparator(Frame(),"separator",args,argcount); 
	XtManageChild(m_wdgSeparator);
    }
}

CDialog::CDialogButton::CDialogButton(CDialog *pDialog, Widget wdgParent, int nId)
    : CButton(wdgParent, nId)
{
    m_pDialog = pDialog;
}

void CDialog::CDialogButton::OnPushed(int nId)
{
    switch ( nId )
    {
	case IDOK:
	    m_pDialog->OnOk();
	    break;
	case IDCANCEL:
	    m_pDialog->OnCancel();
	    break;
    }
}

CDialog::~CDialog()
{
}

void CDialog::InternalDestroy()
{
    if ( m_OkButton != NULL )
    {
	delete m_OkButton;
	m_OkButton = NULL;
    }
	
    if ( m_CancelButton != NULL )
    {
	delete m_CancelButton;
	m_CancelButton = NULL;
    }

    CGuiElement::InternalDestroy();
}

void CDialog::OnOk()
{
    //Destroy();
	
    m_nReturnValue = IDOK;
}

void CDialog::OnYes()
{
    m_nReturnValue = IDYES;
}

void CDialog::OnNo()
{
    m_nReturnValue = IDNO;
}

void CDialog::OnCancel()
{
    //Destroy();
	
    m_nReturnValue = IDCANCEL;
}

const char *CDialog::Caption()
{
    return (const char *)m_strCaption;
}

void CDialog::SetCaption(const char *strCaption)
{
    m_strCaption = strCaption;
	
    Arg args[5];
    Cardinal argcount = 0;
	
    XtSetArg(args[argcount],XmNtitle, strCaption); argcount++;
	
    XtSetValues(Container(), args, argcount);
}

void CDialog::Center()
{
    XWindowAttributes Attributes;

    Display *d = XtDisplay(Parent());

    int screen = DefaultScreen(d);
    Window root = RootWindow(d, screen);

    XGetWindowAttributes(d, root, &Attributes);
    
    int nX = 0;
    int nY = 0;
    int nWidth = 0;
    int nHeight = 0;

    Dimensions(nX, nY, nWidth, nHeight);

    nX = (Attributes.width-nWidth)/2;
    nY = (Attributes.height-nHeight)/2;

    SetDimensions(nX, nY, -1, -1);
}



