/*******************************************************************************
 *
 * libgui Version 0.0.1
 *
 * libgui is a simple framework of GUI classes for X11/Motif widgets written in C++.
 *
 * The full text of the GNU General Public License can be found in the doc 
 * directory.
 *
 * Copyright (C) 2002, 2003 Stefan Hoefer
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * You can contact Stefan Hoefer via email at stefan@hoefer.ch
 *
 ******************************************************************************/

#ifndef DIALOG_H

#define DIALOG_H

#include "guielement.h"
#include "button.h"

#define DLG_STYLE_OK            1L
#define DLG_STYLE_CANCEL        2L
#define DLG_STYLE_SEPARATOR     4L
#define DLG_STYLE_DEFAULT       0L
#define DLG_STYLE_MODAL_DEFAULT 7L

// return values of the dialog

class CDialog : public CGuiElement
{
 private:
    Widget m_wdgShell;
    Widget m_wdgSeparator;
    //Widget m_wdgOkButton;
    //Widget m_wdgCancelButton;
    int m_nReturnValue;
    CStrings m_strCaption;
		
    class CDialogButton : public CButton
	{
	private:
	    CDialog *m_pDialog;
	protected:
	    void OnPushed(int nId);
	public:
	    CDialogButton(CDialog *pDialog, Widget wdgParent, int nId);
	};
		
    friend class CDialogButton;
		
    CDialogButton *m_OkButton;
    CDialogButton *m_CancelButton;
 protected:
    inline void SetContainer(Widget wdgShell) { m_wdgShell = wdgShell; }
    virtual void InternalCreate(unsigned long lStyle);
    virtual void InternalDestroy();
    inline CDialogButton *OkButton() { return m_OkButton; }
    inline CDialogButton *CancelButton() { return m_CancelButton; }
    virtual void OnOk();
    virtual void OnCancel();
    virtual void OnYes();
    virtual void OnNo();
    inline int ReturnValue() { return m_nReturnValue; }
    inline void SetReturnValue(int nReturnValue) { m_nReturnValue = nReturnValue; }
 public:
    enum {IDOK=-1, IDCANCEL=-2, IDYES=-3, IDNO=-4};
    virtual Widget Container() const;
    virtual Widget Frame()=NULL;
    inline Widget Separator() { return m_wdgSeparator; }
    void SetCaption(const char *strCaption);
    const char *Caption();
    void Center();

    CDialog(long lStyle = DLG_STYLE_DEFAULT);		
    ~CDialog();
};

#endif








