/*******************************************************************************
 *
 * libgui Version 0.0.1
 *
 * libgui is a simple framework of GUI classes for X11/Motif widgets written in C++.
 *
 * The full text of the GNU General Public License can be found in the doc 
 * directory.
 *
 * Copyright (C) 2002, 2003 Stefan Hoefer
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * You can contact Stefan Hoefer via email at stefan@hoefer.ch
 *
 ******************************************************************************/

#include "filebutton.h"
#include <basic/macrodefinitions.h>
#include "fileselectiondialog.h"

CFileButton::CFileButton()
{

}

CFileButton::CFileButton(Widget wdgParent, const char *strPath)
: CButton(wdgParent, 0)
{
    if (strPath != NULL)
    {
	m_strPath = strPath;
    }
}

void CFileButton::OnPushed(int nId)
{
    ASSERT(nId == 0);
	
    if (Style() & FB_STYLE_SELECTPATH)
    {
	//CFileSelectionDialog dlgFile(Parent(),DLG_STYLE_DEFAULT);
	CFileSelectionDialog *dlgFile = CreateDialog(DLG_STYLE_DEFAULT);
		
	dlgFile->Create();
	dlgFile->SetCurrentDirectory(Path());
	
	if (dlgFile->DoModal() == CDialog::IDOK)
	{
	    if (dlgFile->Path() != "")
	    {
		SetPath(dlgFile->Path());
		SetFile("");
		
		SendMessage(EVENT_MASK_VALUE_UPDATE);
	    }
	}		

	delete dlgFile;
    }
    else
    {
	//CFileSelectionDialog dlgFile(Parent(),DLG_STYLE_DEFAULT);
	CFileSelectionDialog *dlgFile = CreateDialog(DLG_STYLE_DEFAULT);
		
	dlgFile->Create();
	dlgFile->SetCurrentDirectory(Path());
		
	if (dlgFile->DoModal() == CDialog::IDOK)
	{
	    if (dlgFile->Path() != "")
	    {
		SetPath(dlgFile->Path());
		SetFile(dlgFile->File());
		
		SendMessage(EVENT_MASK_VALUE_UPDATE);
	    }
	}		

	delete dlgFile;
    }
}

CFileSelectionDialog *CFileButton::CreateDialog(unsigned long lStyle)
{
    return new CFileSelectionDialog(DLG_STYLE_DEFAULT);
}

void CFileButton::SetPath(const char *strPath)
{
    m_strPath = strPath;
	
    if (Created())
    {
	SetCaption(m_strPath+m_strFile);
    }
}

void CFileButton::SetFile(const char *strFile)
{
    CStrings strBuffer = strFile;

    int nPosition = strBuffer.FindReverse('/');

    if (nPosition < 0)
    {
	m_strFile = strFile;
    }
    else
    {
	m_strFile = strBuffer.Mid(nPosition+1);
	m_strPath = strBuffer.Left(nPosition+1);
    }

    if (Created())
    {
	SetCaption(m_strPath+m_strFile);
    }
}

void CFileButton::InternalCreate(unsigned long lStyle)
{
    CButton::InternalCreate(lStyle);
	
    Arg args[5];
    Cardinal argcount = 0;
	
    XtSetArg(args[argcount],XmNalignment, XmALIGNMENT_END); argcount++;
	
    XtSetValues(Container(), args, argcount); 
	
    SetCaption(m_strPath);
}


