/*******************************************************************************
 *
 * libgui Version 0.0.1
 *
 * libgui is a simple framework of GUI classes for X11/Motif widgets written in C++.
 *
 * The full text of the GNU General Public License can be found in the doc 
 * directory.
 *
 * Copyright (C) 2002, 2003 Stefan Hoefer
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * You can contact Stefan Hoefer via email at stefan@hoefer.ch
 *
 ******************************************************************************/

//#include "gui.h"
#include <X11/IntrinsicP.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Intrinsic.h>
#include <Xm/ScrolledW.h>
#include <Xm/DrawingA.h>
#include <Xm/Form.h>
#include <Xm/PushB.h>
#include <Xm/RowColumn.h>
#include <Xm/ArrowB.h>
#include "graphicallist.h"
#include <basic/iterator.h>

#include <basic/cstrings.h>
#include <Xm/ToggleB.h>

bool graphical_list_box::m_bActionsAdded = false;
bool graphical_list_item::m_bActionsAdded = false;
XtActionsRec graphical_list_box::m_arrActions[] = {{"GLBHandleTab",graphical_list_box::TabHandler}};
XtActionsRec graphical_list_item::m_arrActions[] = {{"GLIHandleTab",graphical_list_item::TabHandler}};
XtTranslations graphical_list_box::m_Translations;
XtTranslations graphical_list_item::m_Translations;

void graphical_list_item::sublisthandle(Widget w, XtPointer client_data, XmAnyCallbackStruct *call_data)
{
    graphical_list_item *item;
    
    item = (graphical_list_item *)client_data;
    
    if(item->is_visible())
    {
	item->invisible();
    }
    else
    {
	item->visible();
    }

    item->m_box->SetInputFocus();
    item->m_box->OnOpenNode(item);
}

/* 
void graphical_list_item::MouseHandler(Widget widg, XtPointer client_data, XEvent *p_event)
{
}

bool graphical_list_box::ContextMenu(graphical_list_item *pItem)
{
    return false;
}

void graphical_list_box::CreateContextMenu(graphical_list_item *pItem, CPopupMenu& PopupMenu)
{

}

void graphical_list_box::NotifyContextMenu(int nResult)
{

}
*/

void graphical_list_box::TabHandler(Widget w, XEvent *event, String *params, Cardinal *num_params)
{
    Arg args[5];
    Cardinal argcount = 0;
    XtPointer pointer;

    //... Fehler: Nicht alle Widgets, die diesen Handler aufrufen, haben die Treebox als userData gespeichert!
    //		    Behebung: eventuell zweiten Handler fr items.
    // --> Behoben.

    XtSetArg(args[argcount],XmNuserData,&pointer); argcount++;
	
    XtGetValues(w,args,argcount);
    
    CGraphicalListBoxContext *pContext = (CGraphicalListBoxContext *)pointer;
    ASSERT(pContext->Type() == CGraphicalObjectContext::GRAPHICAL_LIST_BOX);

    //graphical_list_box *pBox = (graphical_list_box *)pointer;
    graphical_list_box *pBox = pContext->Object();
	
    pBox->OnTab();
}

void graphical_list_item::TabHandler(Widget w, XEvent *event, String *params, Cardinal *num_params)
{
    Arg args[5];
    Cardinal argcount = 0;
    XtPointer pointer;

    //... Fehler: Nicht alle Widgets, die diesen Handler aufrufen, haben die Treebox als userData gespeichert!
    //		    Behebung: eventuell zweiten Handler fr items.
    // --> Behoben.

    XtSetArg(args[argcount],XmNuserData,&pointer); argcount++;
	
    XtGetValues(w,args,argcount);
	
    CGraphicalListItemContext *pContext = (CGraphicalListItemContext *)pointer;
    ASSERT(pContext->Type() == CGraphicalObjectContext::GRAPHICAL_LIST_ITEM);

    graphical_list_item *pItem = pContext->Object();
	
    pItem->m_box->OnTab();
}

void graphical_list_box::OnTab()
{

}

graphical_list_item::graphical_list_item(Widget left, Widget top, const char *t, graphical_list_box *box, int nPosition)
    : m_Context(this)
{
    if ( !m_bActionsAdded )
    {
	m_Translations = XtParseTranslationTable("<Key>Tab: GLIHandleTab()");
	XtAppAddActions(box->ApplicationContext(),m_arrActions,XtNumber(m_arrActions));

	m_bActionsAdded = true;
    }

    m_wdgCheckBox = NULL;

    m_box = box;
	
    Arg args[20];
    Cardinal argcount=0;
	
    if ( nPosition >= 0 )
    {
	XtSetArg(args[argcount],XmNpositionIndex,nPosition); argcount++;
    }
    //XtSetArg(args[argcount],XmNbackground,(long)0xffff); argcount++; //!!!
		
    myform = XmCreateForm(left,"topform", args, argcount);
    XtManageChild(myform);
    if (box->HasContextMenu())
    {
	box->RegisterPopupCaller(myform, TOP_POPUP_MENU);
    }

    //myform = XtVaCreateManagedWidget("form",
    //			   xmFormWidgetClass,left,
    /*XmNleftAttachment,XmATTACH_FORM,
      XmNtopAttachment,XmATTACH_WIDGET,
      XmNtopWidget,top,*/
    //			   NULL);
	
    mybuttonform = XtVaCreateManagedWidget("form",
					   xmFormWidgetClass,myform,
					   XmNleftAttachment,XmATTACH_FORM,
					   XmNtopAttachment,XmATTACH_FORM,
					   XmNbottomAttachment,XmATTACH_FORM,
					   XmNwidth,20,
					   NULL);
	
    /*mylabel = XtVaCreateManagedWidget(t,
      xmLabelWidgetClass,myform,
      XmNleftAttachment,XmATTACH_WIDGET,
      XmNleftWidget,mybuttonform,
      XmNtopAttachment,XmATTACH_FORM,
      // XmNbackground,0xf800,
      NULL);*/
	
    Widget leftWidget = NULL;

    if (box->Style() & HIERARCHICAL_CHECK_BOX)
    {
	argcount = 0;
	
	XtSetArg(args[argcount],XmNleftAttachment,XmATTACH_WIDGET); argcount++;
	XtSetArg(args[argcount],XmNleftWidget,mybuttonform); argcount++;
	XtSetArg(args[argcount],XmNtopAttachment,XmATTACH_FORM); argcount++;
	XtSetArg(args[argcount],XmNindicatorOn,XmINDICATOR_CHECK_BOX); argcount++;
	XtSetArg(args[argcount],XmNspacing, 0); argcount++;
	XtSetArg(args[argcount],XmNmarginWidth,0); argcount++;
	XtSetArg(args[argcount],XmNmarginHeight,0); argcount++;
	XtSetArg(args[argcount],XmNshadowThickness,0); argcount++;
	XtSetArg(args[argcount],XmNindicatorSize,15); argcount++;
	XtSetArg(args[argcount],XmNtopOffset,-2); argcount++;
	XtSetArg(args[argcount],XmNuserData, &m_Context); argcount++;
	m_wdgCheckBox = XmCreateToggleButton(myform, "", args, argcount);
	XtManageChild(m_wdgCheckBox);
	leftWidget = m_wdgCheckBox;

	XtAddCallback(leftWidget, XmNvalueChangedCallback, (XtCallbackProc)graphical_list_box::HandleItemChecked, (XtPointer)this);
    }
    else
    {
	leftWidget = mybuttonform;
    }

    argcount = 0;
    XtSetArg(args[argcount],XmNleftAttachment,XmATTACH_WIDGET); argcount++;
    XtSetArg(args[argcount],XmNleftWidget,leftWidget); argcount++;
    XtSetArg(args[argcount],XmNtopAttachment,XmATTACH_FORM); argcount++;

    char *strBuffer = new char[strlen(t)+1];
    strcpy(strBuffer,t);
	
    XmString strLabel = XmStringCreate(strBuffer,XmFONTLIST_DEFAULT_TAG);

    delete strBuffer;
	
    XtSetArg(args[argcount],XmNlabelString,strLabel); argcount++;
    XtSetArg(args[argcount],XmNmarginWidth,0); argcount++;
    XtSetArg(args[argcount],XmNmarginHeight,0); argcount++;
    XtSetArg(args[argcount],XmNfillOnArm,False); argcount++;
    XtSetArg(args[argcount],XmNshadowThickness,0); argcount++;
    //XtSetArg(args[argcount],XmNuserData,(XtPointer)this); argcount++;
    XtSetArg(args[argcount],XmNuserData, (XtPointer)&m_Context); argcount++;
    //XtSetArg(args[argcount],XmNmultiClick, XmMULTICLICK_KEEP); argcount++;
		
    mylabel = XmCreatePushButton(myform,"button",args,argcount);
    XtManageChild(mylabel);
    //XtAddEventHandler(mylabel,ButtonPressMask,FALSE,(void (*)(_WidgetRec *, void *, _XEvent *, char *))MouseHandler,this);		
    XtOverrideTranslations(mylabel, m_Translations);

    XmStringFree(strLabel);
	
    /*mysubform = XtVaCreateManagedWidget("form",
      xmFormWidgetClass,myform,
      XmNleftAttachment,XmATTACH_FORM,
      XmNtopAttachment,XmATTACH_WIDGET,
      XmNtopWidget,mylabel,
      XmNhorizontalSpacing,20,
      NULL);*/
				      
    argcount = 0;
    XtSetArg(args[argcount],XmNleftAttachment,XmATTACH_WIDGET); argcount++;
    XtSetArg(args[argcount],XmNtopAttachment,XmATTACH_WIDGET); argcount++;
    XtSetArg(args[argcount],XmNtopWidget,mylabel); argcount++;
    XtSetArg(args[argcount],XmNleftWidget,mybuttonform); argcount++;
    XtSetArg(args[argcount], XmNrightAttachment, XmATTACH_FORM); argcount++;
    XtSetArg(args[argcount], XmNbottomAttachment, XmATTACH_FORM); argcount++;
    XtSetArg(args[argcount],XmNspacing,0); argcount++;
    //XtSetArg(args[argcount],XmNpacking,XmPACK_COLUMN); argcount++;
    XtSetArg(args[argcount],XmNmarginWidth,0); argcount++;
    XtSetArg(args[argcount],XmNmarginHeight,0); argcount++;
    //XtSetArg(args[argcount],XmNuserData, (XtPointer)this); argcount++;
    XtSetArg(args[argcount],XmNuserData, (XtPointer)&m_Context); argcount++;
    mysubform = XmCreateRowColumn(myform,"subform",args,argcount);
    //XtManageChild(mysubform);
    XtOverrideTranslations(mysubform, m_Translations);
    if (box->HasContextMenu())
    {
	box->RegisterPopupCaller(mysubform, TOP_POPUP_MENU);
    }

    mybutton = 0;
    visible_flag|=0x1;

    create_sub_button();
    ShowSubButton(false);
  	
    m_pData = NULL;
} 

void graphical_list_item::Check(bool bCheck)
{
    if (HBox()->Style() & HIERARCHICAL_CHECK_BOX)
    {
	if (m_wdgCheckBox != NULL)
	{
//	    XmToggleButtonSetValue(m_wdgCheckBox, bCheck?True:False, False);
	    XmToggleButtonSetState(m_wdgCheckBox, bCheck?True:False, False);
	}
    }
}

void graphical_list_item::disappear()
{
    // Make sure that the parents will recompute their size!
	
    iterator<graphical_list_item *> it_it;
    graphical_list_item *item;

    if(is_visible())
    {
    	XtUnmanageChild(mysubform);
    	it_it << dynamic_cast<basic_list<graphical_list_item *> *>(this);
    	while(item = it_it++)
    	{
	    item->disappear();
    	}
    }

    XtUnmanageChild(mylabel);
    XtUnmanageChild(myform);
    if(mybutton)
    {
    	XtUnmanageChild(mybutton);
    }
    XtUnmanageChild(mybuttonform);
    //XtUnmanageChild(Lines());
}

void graphical_list_box::OnOpenNode(graphical_list_item *pItem)
{

}

void graphical_list_item::appear()
{
    iterator<graphical_list_item *> it_it;
    graphical_list_item *item;

    if(is_visible())
    {
    	XtManageChild(mysubform);
    	it_it << dynamic_cast<basic_list<graphical_list_item *> *>(this);
    	while(item = it_it++)
    	{
	    item->appear();
    	}
    }

    XtManageChild(myform);
    XtManageChild(mylabel);
    XtManageChild(mybuttonform);
    XtManageChild(mybutton);

    if ( card() == 0 )
    {
	ShowSubButton(false);		
    }

    //XtManageChild(Lines());
}

graphical_list_item *graphical_list_item::add_item(const char *t, int nPosition)
{
    graphical_list_item *l;
  
    if(!card())
    {
    	//create_sub_button();
	ShowSubButton();
    	*this += l = new_item(mysubform,mysubform,t,nPosition);
    }
    else
    {
    	*this += l = new_item(mysubform,last()->myform,t,nPosition);        
    }
    l->parent = this;

    return l;
}

graphical_list_item *graphical_list_item::new_item(Widget left,Widget top,const char *t, int nPosition)
{
    graphical_list_item *pItem = new graphical_list_item(left,top,t, HBox(), nPosition);

    if (HBox()->HasContextMenu())
    {
	HBox()->AttachPopupMenu(pItem->mylabel);
    }

    return pItem;
}

void graphical_list_item::create_sub_button()
{
    Arg args[20];
    Cardinal argcount = 0;
	
    mybutton = XtVaCreateManagedWidget("H",
				       xmArrowButtonWidgetClass,mybuttonform,
				       XmNleftAttachment,XmATTACH_FORM,
				       XmNtopAttachment,XmATTACH_FORM,
				       XmNshadowThickness,0,
				       XmNheight,18,
				       XmNarrowDirection,XmARROW_DOWN,
				       //XmNuserData, (XtPointer)this,
				       XmNuserData, (XtPointer)&m_Context,
				       NULL);
    XtAddCallback(mybutton,XmNdisarmCallback,(XtCallbackProc)sublisthandle,(XtPointer)this);
  
    XtOverrideTranslations(mybutton, m_Translations);

    argcount = 0;
    XtSetArg(args[argcount],XmNtopAttachment, XmATTACH_WIDGET); argcount++;
    XtSetArg(args[argcount],XmNtopWidget, mybutton); argcount++;
    XtSetArg(args[argcount],XmNleftAttachment, XmATTACH_FORM); argcount++;
    XtSetArg(args[argcount],XmNrightAttachment, XmATTACH_FORM); argcount++;
    XtSetArg(args[argcount],XmNbottomAttachment, XmATTACH_FORM); argcount++;
    //XtSetArg(args[argcount],XmNuserData, this); argcount++;
    XtSetArg(args[argcount],XmNuserData, (XtPointer)&m_Context); argcount++;
    //XtSetArg(args[argcount],XmNbackground,(long)0xF); argcount++;
  	
    m_wdgLines = XmCreateDrawingArea(mybuttonform, "lines", args, argcount);
    XtManageChild(m_wdgLines);
}

void graphical_list_item::ShowSubButton(bool bShow)
{
    if ( bShow )
    {
	XtMapWidget(mybutton);
	XtManageChild(mysubform);
    }
    else
    {
	XtUnmapWidget(mybutton);
	XtUnmanageChild(mysubform);
    }
}

void graphical_list_item::visible() 
{  	
    if (card() > 0)
    {
	XtManageChild(Lines());
	XtManageChild(mysubform);

	Arg args[20];
	int n;
	iterator<graphical_list_item *> it_it;
	graphical_list_item *item;

	it_it << dynamic_cast<basic_list<graphical_list_item *> *>(this);
	while(item = it_it++)
	{
	    item->appear();
	}

	visible_flag |= 0x1;
	n=0;
	XtSetArg(args[n],XmNarrowDirection,XmARROW_DOWN); n++;
	XtSetValues(mybutton,args,n);
    }
}

int graphical_list_item::is_visible_on_screen()
{
    graphical_list_item *item;

    item = parent;
    while(item){
	if(!item->is_visible())
	    return 0;
	item = item->get_parent();
    }
    return 1;
}

void graphical_list_item::invisible() 
{
    Arg args[20];
    int n;
    iterator<graphical_list_item *> it_it;
	
    XtUnmanageChild(Lines());
    XtUnmanageChild(mysubform);

    graphical_list_item *item;

    it_it << dynamic_cast<basic_list<graphical_list_item *> *>(this);
    while(item = it_it++)
    {
    	item->disappear();
    }

    visible_flag &=0x0;
    n=0;
    XtSetArg(args[n],XmNarrowDirection,XmARROW_RIGHT); n++;
    XtSetValues(mybutton,args,n);
}

graphical_list_box::graphical_list_box(Widget p, int nId)
    : CGuiManager(p, 0L, nId), m_Context(this)
{
    if ( !m_bActionsAdded )
    {
	m_Translations = XtParseTranslationTable("<Key>Tab: GLBHandleTab()");
	XtAppAddActions(ApplicationContext(),m_arrActions,XtNumber(m_arrActions));

	m_bActionsAdded = true;
    }
}

void graphical_list_box::InternalCreate(unsigned long lStyle)
{
    Arg args[20];
    Cardinal argcount;

    //ASSERT(m_bActionsAdded);

    argcount = 0;
    XtSetArg(args[argcount], XmNmarginWidth,0); argcount++;
    XtSetArg(args[argcount], XmNmarginHeight,0); argcount++;
    //XtSetArg(args[argcount], XmNuserData, (XtPointer)this); argcount++;
    XtSetArg(args[argcount], XmNuserData, (XtPointer)&m_Context); argcount++;

    m_DrawingArea = XmCreateDrawingArea(Parent(),"da",args,argcount);
    XtManageChild(m_DrawingArea);
    XtAddCallback(m_DrawingArea,XmNresizeCallback,(XtCallbackProc)resize_handler,(XtPointer)this);  
    XtAddCallback(m_DrawingArea,XmNinputCallback,(XtCallbackProc)InputHandler,(XtPointer)this);  

    XtOverrideTranslations(m_DrawingArea, m_Translations);

    argcount = 0;
    XtSetArg(args[argcount],XmNinitialFocus,m_DrawingArea); argcount++;
    XtSetValues(m_DrawingArea,args,argcount); 
	
    argcount = 0;
    XtSetArg(args[argcount], XmNscrollingPolicy,XmAUTOMATIC); argcount++;
    XtSetArg(args[argcount], XmNtraversalOn,FALSE); argcount++;
    //XtSetArg(args[argcount], XmNuserData, this); argcount++;
    XtSetArg(args[argcount], XmNuserData, (XtPointer)&m_Context); argcount++;

    m_ScrolledWindow = XmCreateScrolledWindow(m_DrawingArea,"scrolledwindow",args,argcount);
	
    XtManageChild(m_ScrolledWindow);
    XtOverrideTranslations(m_ScrolledWindow, m_Translations);

    Widget bar1, bar2;
	
    argcount = 0;
    XtSetArg(args[argcount],XmNhorizontalScrollBar,&bar1); argcount++;
    XtSetArg(args[argcount],XmNverticalScrollBar,&bar2); argcount++;
    XtGetValues(m_ScrolledWindow,args,argcount);
    XtAddCallback(bar1,XmNdragCallback,(XtCallbackProc)ScrollHandler,(XtPointer)this);  
    XtAddCallback(bar2,XmNdragCallback,(XtCallbackProc)ScrollHandler,(XtPointer)this);  
    XtAddCallback(bar1,XmNincrementCallback,(XtCallbackProc)ScrollHandler,(XtPointer)this);  
    XtAddCallback(bar2,XmNincrementCallback,(XtCallbackProc)ScrollHandler,(XtPointer)this);  
    
    argcount = 0;
    myform = XmCreateRowColumn(m_ScrolledWindow,"topform",args,argcount);	
    //RegisterPopupManager(myform);

    XtManageChild(myform);
    
    if (HasContextMenu())
    {
	CSimpleObjectList<CIdString> Items;
	CSimpleObjectList<CIdString> TopItems;

	CreateContextMenu(Items);
	CreateTopContextMenu(TopItems);

	AddPopupMenu(Items, STANDARD_POPUP_MENU);
	AddPopupMenu(TopItems, TOP_POPUP_MENU);

	AttachTopPopupMenu();

	//RegisterPopupCaller(m_DrawingArea, TOP_POPUP_MENU);
	//RegisterPopupCaller(m_ScrolledWindow, TOP_POPUP_MENU);
	RegisterPopupCaller(myform, TOP_POPUP_MENU);
    }

    SetInputFocus();
}

void graphical_list_box::AttachTopPopupMenu()
{
    AttachPopupMenu(m_ScrolledWindow, TOP_POPUP_MENU);
}

void graphical_list_box::AttachPopupMenu(Widget wdgButton, int nMenuId)
{
    CPopupMenuHandle *pHandle = PopupMenu(nMenuId);

    if (pHandle != NULL)
    {
	pHandle->AddPopup(wdgButton);
    }
    else
    {
	TRACE("Could not find Menu Handle!");
    }
}

void graphical_list_box::AttachPopupMenu(Widget wdgButton)
{
    AttachPopupMenu(wdgButton, STANDARD_POPUP_MENU);
}

graphical_list_item *graphical_list_box::add_item(const char *t, int nPosition)
{
    graphical_list_item *l;
  
    if(!card())
    {
    	*this += l = new_item(myform,myform,t,nPosition);
    }
    else
    {
    	*this += l = new_item(myform,myform,t,nPosition);
    }

    l->parent = (graphical_list_item *)NULL;

    return l;
}

graphical_list_item *graphical_list_box::new_item(Widget left,Widget top,const char *t, int nPosition)
{
    graphical_list_item *pItem = new graphical_list_item(left,top,t, this, nPosition);

    if (HasContextMenu())
    {
	AttachPopupMenu(pItem->mylabel);
    }

    return pItem;
}

Widget graphical_list_box::Container() const
{ 
    return m_DrawingArea; 
}

bool graphical_list_box::HasContextMenu()
{
    return false;
}

void graphical_list_box::CreateContextMenu(CSimpleObjectList<CIdString> &Items)
{
 
}

void graphical_list_box::CreateTopContextMenu(CSimpleObjectList<CIdString> &Items)
{

}

/*!
*
* \fn void graphical_list_box::DeleteAllItems()
*
* \author STH
* \date 23.4.00
*
*/

void graphical_list_box::DeleteAllItems()
{				   
    graphical_list_item *pItem;
    iterator<graphical_list_item *> ItemIt;
		
    ItemIt << this;
	
    while ( pItem = ItemIt++ )
    {
	delete pItem;
    }
	
    reset();	
}

/*!
*
* \fn void graphical_list_item::RemoveFromList()
*
* \author STH
* \date 24.4.00
*
*/

void graphical_list_item::RemoveFromList()
{
    // Delete subitems
	
    if (card())
    {
	graphical_list_item *pItem;
		
	while (pItem = operator[](0))
	{
	    pItem->RemoveFromList();
	}	
    }

    graphical_list_item *pParent = get_parent();
	
	// Remove the actual list item from the view
	
    XtDestroyWidget(myform);
	
    if ( pParent != NULL )
    {
	*pParent -= this;
    }
	
    delete this;
	
	// If this was the last subitem, adapt the parent.

    if (pParent != NULL && pParent->card() == 0)
    {
	/*XtDestroyWidget(pParent->mybutton);
	  pParent->mybutton = NULL;*/
		
	pParent->ShowSubButton(false);
	Arg args[5];
	Cardinal argcount;
		
	argcount = 0;
	XtSetArg(args[argcount],XmNwidth,20); argcount++;
		
	XtSetValues(pParent->mybuttonform,args,argcount);
    }
}

graphical_list_item::~graphical_list_item()
{
    iterator<graphical_list_item *> ItemIt;
    graphical_list_item *pItem;
	
    XtUnmapWidget(myform);
	
    ItemIt << this;
    while ( pItem = ItemIt++ ){
	delete pItem;
    }
	
    reset();
    XtDestroyWidget(myform);
}

graphical_list_box::~graphical_list_box()
{
    XtDestroyWidget(Container());
}

void graphical_list_box::SetInputFocus()
{
    XmProcessTraversal(m_DrawingArea, XmTRAVERSE_CURRENT);
}

void graphical_list_box::AdaptSize()
{
    Dimension Width,Height;
    Arg args[5];
    Cardinal n;

    n = 0;
    XtSetArg(args[n],XmNwidth,&Width); n++;
    XtSetArg(args[n],XmNheight,&Height); n++;
    XtGetValues(Container(),args,n);

    n = 0;
    XtSetArg(args[n],XmNwidth,Width); n++;
    XtSetArg(args[n],XmNheight,Height); n++;
    XtSetValues(m_ScrolledWindow,args,n);
}

void graphical_list_box::resize_handler(Widget w, XtPointer client_data, XmDrawingAreaCallbackStruct *call_data)
{
    graphical_list_box *pBox = (graphical_list_box *)client_data;

    pBox->AdaptSize();

    pBox->SetInputFocus();
}

void graphical_list_box::ScrollHandler(Widget w, XtPointer client_data, XmAnyCallbackStruct *call_data)
{
    graphical_list_box *pBox = (graphical_list_box *)client_data;

    if (pBox->NotifyScroll())
    {
	pBox->SetInputFocus();
    }
}

void graphical_list_box::HandleItemChecked(Widget w, XtPointer client_data, XmToggleButtonCallbackStruct *call_data)
{
    /*Arg args[5];
    Cardinal argcount = 0;
    XtPointer pointer;

    XtSetArg(args[argcount], XmNuserData, &pointer); argcount++;

    XtGetValues(w, args, argcount);

    CGraphicalListItemContext *pContext = (CGraphicalListItemContext *)pointer;

    ASSERT(pContext->Type() == CGraphicalObjectContext::GRAPHICAL_LIST_ITEM);
    graphical_list_item *pItem = pContext->Object();*/

    graphical_list_item *pItem = (graphical_list_item *)client_data;

    bool bSet = (bool)call_data->set;

    pItem->HBox()->OnItemChecked(pItem, bSet);
}

void graphical_list_box::OnItemChecked(graphical_list_item *pItem, bool bSet)
{

}

void graphical_list_box::InputHandler(Widget w, XtPointer client_data, XmDrawingAreaCallbackStruct *call_data)
{
    graphical_list_box *pBox = (graphical_list_box *)client_data;

    if (pBox->NotifyInput(call_data))
    {
	pBox->SetInputFocus();
    }    
}

bool graphical_list_box::NotifyScroll()
{
    return true;
}

bool graphical_list_box::NotifyInput(XmDrawingAreaCallbackStruct *call_data)
{
    return true;
}

CGraphicalObjectContext::GOC CGraphicalListItemContext::Type()
{
    return GRAPHICAL_LIST_ITEM;
}

CGraphicalListItemContext::CGraphicalListItemContext(graphical_list_item *pItem)
    : CGraphicalObjectContext((void *)pItem)
{

}

CGraphicalObjectContext::GOC CGraphicalListBoxContext::Type()
{
    return GRAPHICAL_LIST_BOX;
}

CGraphicalListBoxContext::CGraphicalListBoxContext(graphical_list_box *pBox)
    : CGraphicalObjectContext((void *)pBox)
{

}


