/*******************************************************************************
 *
 * libgui Version 0.0.1
 *
 * libgui is a simple framework of GUI classes for X11/Motif widgets written in C++.
 *
 * The full text of the GNU General Public License can be found in the doc 
 * directory.
 *
 * Copyright (C) 2002, 2003 Stefan Hoefer
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * You can contact Stefan Hoefer via email at stefan@hoefer.ch
 *
 ******************************************************************************/

#ifndef GRAPHICALLIST_H

#define GRAPHICALLIST_H

#include <Xm/Xm.h>
#include <basic/basiclist.h>
#include "guimanager.h"
#include "graphicalobjectcontext.h"

//-----------------------------------------------------------------------
// graphical hierarchical lists
//-----------------------------------------------------------------------

static void sublisthandle(Widget , XtPointer , XmAnyCallbackStruct *);

class graphical_list_box;
class graphical_list_item;

class CGraphicalListItemContext: public CGraphicalObjectContext
{
 public:
    virtual GOC Type();
    CGraphicalListItemContext(graphical_list_item *pItem);
    inline graphical_list_item *Object() { return (graphical_list_item *)CGraphicalObjectContext::Object(); }
};

class CGraphicalListBoxContext: public CGraphicalObjectContext
{
 public:
    virtual GOC Type();
    CGraphicalListBoxContext(graphical_list_box *pItem);
    inline graphical_list_box *Object() { return (graphical_list_box *)CGraphicalObjectContext::Object(); }
};

#define STANDARD_POPUP_MENU 999999
#define TOP_POPUP_MENU      999998

#define HIERARCHICAL_NEW_ITEM        1L
#define HIERARCHICAL_LINES           2L
#define HIERARCHICAL_STANDARD_POPUP  4L
#define HIERARCHICAL_MULTI_SELECT    8L
#define HIERARCHICAL_CHECK_BOX      16L
#define HIERARCHICAL_READONLY       32L

class graphical_list_item : public basic_list<graphical_list_item *>
{
    friend class graphical_list_box;
 private:
    unsigned int visible_flag:1;
    graphical_list_item *parent;
    void *m_pData;
    Widget m_wdgLines;
    graphical_list_box *m_box;
    static bool m_bActionsAdded;
    static XtActionsRec m_arrActions[];
    static XtTranslations m_Translations;
    CGraphicalListItemContext m_Context;
    Widget m_wdgCheckBox;
 private:
    //static void MouseHandler(Widget widg, XtPointer client_data, XEvent *p_event);	
    inline graphical_list_box *HBox() { return m_box; }
    static void sublisthandle(Widget w, XtPointer client_data, XmAnyCallbackStruct *call_data);
    static void TabHandler(Widget w, XEvent *event, String *params, Cardinal *num_params);
 protected:
    virtual graphical_list_item *new_item(Widget left,Widget top,const char *t, int nPosition=-1);
 public:
    Widget myform;
    Widget mylabel;
    Widget mybuttonform;
    Widget mybutton;
    Widget mysubform;
		  
    inline Widget get_mysubform() { return mysubform;}
    inline Widget get_myform() { return myform;}
    inline graphical_list_item *get_parent() {return parent;}
    void visible();
    void invisible();
    inline int is_visible() {return visible_flag&0x1;}
    int is_visible_on_screen();

    graphical_list_item(Widget, Widget, const char *, graphical_list_box *box, int nPosition=-1);
    virtual ~graphical_list_item();
  
    void disappear();
    void appear();
    graphical_list_item *add_item(const char *, int nPosition=-1);
    void create_sub_button();
    void RemoveFromList();
	
    void ShowSubButton(bool bShow = true);
		
    inline void SetData(void *pData) { m_pData = pData; }
    inline void *Data() { return m_pData; }
    inline Widget Lines() { return m_wdgLines; }
    void Check(bool bCheck=true);
};

class graphical_list_box : public basic_list<graphical_list_item *>, public CGuiManager
{
    friend class graphical_list_item;
 private:
    Widget myform;
    Widget m_ScrolledWindow;
    Widget m_DrawingArea;

    static bool m_bActionsAdded;
    static XtActionsRec m_arrActions[];
    static XtTranslations m_Translations;
    CGraphicalListBoxContext m_Context;
 private:
    static void resize_handler(Widget w, XtPointer client_data, XmDrawingAreaCallbackStruct *call_data);
    static void ScrollHandler(Widget w, XtPointer client_data, XmAnyCallbackStruct *call_data);
    static void HandleItemChecked(Widget w, XtPointer client_data, XmToggleButtonCallbackStruct *call_data);
    static void InputHandler(Widget w, XtPointer client_data, XmDrawingAreaCallbackStruct *call_data);
    static void TabHandler(Widget w, XEvent *event, String *params, Cardinal *num_params);
	
    void AdaptSize();
 protected:
    virtual graphical_list_item *new_item(Widget left,Widget top,const char *t, int nPosition=-1);
    inline Widget get_myform(){return myform;}
    virtual bool NotifyScroll();
    virtual bool NotifyInput(XmDrawingAreaCallbackStruct *call_data);
    virtual void InternalCreate(unsigned long lStyle=0L);

    inline Widget ScrolledWindow() { return m_ScrolledWindow; }
    virtual bool HasContextMenu();

    virtual void OnTab();
    /*virtual bool ContextMenu(graphical_list_item *pItem);
      virtual void CreateContextMenu(graphical_list_item *pItem, CPopupMenu& PopupMenu);
      virtual void NotifyContextMenu(int nResult);*/

    virtual void CreateContextMenu(CSimpleObjectList<CIdString> &Items);
    virtual void CreateTopContextMenu(CSimpleObjectList<CIdString> &Items);
    void SetInputFocus();
    virtual void OnItemChecked(graphical_list_item *pItem, bool bSet);
    virtual void OnOpenNode(graphical_list_item *pItem);
 public:
    virtual Widget Container() const;	
 protected:
    graphical_list_box(Widget wdgParent, int nId=0);
    ~graphical_list_box();
    graphical_list_item *add_item(const char *, int nPosition = -1);
    void DeleteAllItems();

 private:
    void AttachPopupMenu(Widget wdgButton);
    void AttachTopPopupMenu();
    void AttachPopupMenu(Widget wdgButton, int nMenuId);
};

#endif








