/*******************************************************************************
 *
 * libgui Version 0.0.1
 *
 * libgui is a simple framework of GUI classes for X11/Motif widgets written in C++.
 *
 * The full text of the GNU General Public License can be found in the doc 
 * directory.
 *
 * Copyright (C) 2002, 2003 Stefan Hoefer
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * You can contact Stefan Hoefer via email at stefan@hoefer.ch
 *
 ******************************************************************************/

#include "guielement.h"
#include "xapplication.h"
#include <basic/macrodefinitions.h>
#include <Xm/Xm.h>

CGuiElement::CGuiElement(Widget wdgParent, unsigned long lStyle, int nId)
    : CMessageClient(nId)
{
    m_wdgParent = wdgParent;
    m_wdgContainer = NULL;
    m_lStyle = lStyle;
    SetBeingDestroyed(false);
    SetCreated(false);
    SetContainerDestroyed(false);
    //m_bWorkaroundXtDestroyWidget = false;
}

CGuiElement::CGuiElement(unsigned long lStyle, int nId)
    : CMessageClient(nId)
{
    m_wdgParent = NULL;
    m_wdgContainer = NULL;
    m_lStyle = lStyle;
    SetBeingDestroyed(false);
    SetCreated(false);
    SetContainerDestroyed(false);
    //m_bWorkaroundXtDestroyWidget = false;
}

void CGuiElement::Create(Widget wdgParent, unsigned long lStyle, Arg *pargs, Cardinal pargcount)
{
    if ( Parent() == NULL )
    {
	m_wdgParent = wdgParent;
    }

    Create(lStyle, pargs, pargcount);
}

void CGuiElement::Create(unsigned long lStyle, Arg *pargs, Cardinal pargcount)
{
    if ( ! Created() )
    {
	ASSERT(Parent() != NULL );
		
	m_lStyle |= lStyle;

	InternalCreate(m_lStyle);
	SetCreated(true);
	SetContainerDestroyed(false);
	
	m_wdgContainer = Container();
	if (m_wdgContainer != NULL)
	{
	    XtAddCallback(m_wdgContainer, XmNdestroyCallback, (XtCallbackProc)HandleDestroy, (XtPointer) this);
	}
	
	PostCreate();

	if (pargcount > 0)
	{
	    XtSetValues(Container(), pargs, pargcount);
	}
    }
}

void CGuiElement::HandleDestroy(Widget w, XtPointer client_data, XmAnyCallbackStruct *call_data)
{
    CGuiElement *pElement = (CGuiElement *)client_data;
	
    //pElement->Destroy();

    //pElement->SetCreated(false);

    pElement->SetContainerDestroyed();
}

/*bool CGuiElement::WorkaroundXtDestroyWidget()
{
    return m_bWorkaroundXtDestroyWidget;
}

void CGuiElement::SetWorkaroundXtDestroyWidget()
{
    m_bWorkaroundXtDestroyWidget = true;
}*/

void CGuiElement::Destroy()
{
    if (!BeingDestroyed())
    {
	SetBeingDestroyed(true);
		
	if (Created())
	{
	    InternalDestroy();
		
	    if ((m_wdgContainer != NULL) && !ContainerDestroyed())
	    {
				//XtRemoveAllCallbacks(m_wdgContainer, XmNdestroyCallback);
		//Display *pDisplay = XtDisplay(m_wdgContainer);

		if (XtIsWidget(m_wdgContainer))
		{
		    // Either Intrinsics or Motif seems to have a very serious bug that causes XtDestroyWidget to fail
		    // in certain cases at least with XmComboBox Widgets. This is a dirty workaround.
		    /*if (!WorkaroundXtDestroyWidget())
		      {*/
		    XtDestroyWidget(m_wdgContainer);
		    /*}
		      else
		      {
		      XtUnrealizeWidget(m_wdgContainer);
		      }*/
		}

		m_wdgContainer = NULL;
	    }
			
	    SetCreated(false);
	    SetContainerDestroyed();
	}
		
	SetBeingDestroyed(false);
    }
}

CGuiElement::~CGuiElement()
{
    Destroy();
}

void CGuiElement::SetCaption(const char *strCaption)
{

}

const char *CGuiElement::Caption()
{
    return "";
}

void CGuiElement::SetDimensions(int nX, int nY, int nWidth, int nHeight)
{
    Arg args[10]; 
    Cardinal argcount = 0;
	
    if ( nX >= 0 )
    {
	XtSetArg(args[argcount],XmNx,nX); argcount++;
    }
    if ( nY >= 0 )
    {
	XtSetArg(args[argcount],XmNy,nY); argcount++;
    }
    if ( nWidth >= 0 )
    {
	XtSetArg(args[argcount],XmNwidth,nWidth); argcount++;
    }
    if ( nHeight >= 0 )
    {
	XtSetArg(args[argcount],XmNheight,nHeight); argcount++;
    }
	
    XtSetValues(Container(),args,argcount);
}

void CGuiElement::Dimensions(int &nX, int &nY, int &nWidth, int &nHeight)
{
    Arg args[10]; 
    Cardinal argcount = 0;
	
    XtSetArg(args[argcount],XmNx,&nX); argcount++;
    XtSetArg(args[argcount],XmNy,&nY); argcount++;
    XtSetArg(args[argcount],XmNwidth,&nWidth); argcount++;
    XtSetArg(args[argcount],XmNheight,&nHeight); argcount++;
	
    XtGetValues(Container(),args,argcount);
}

Widget CGuiElement::ApplicationShell(Widget wdgElement)
{
    String strName;

    while(wdgElement != NULL)
    {
	strName = XtName(wdgElement);
		
	if ( strName != NULL && !strcmp(XtName(wdgElement),xapplication::AppShellName()))
	{
	    break;
	}
		
	wdgElement = XtParent(wdgElement);
    }
    
    return wdgElement;
}

XtAppContext CGuiElement::ApplicationContext()
{
//      Widget wdgElement = Parent();
	
//      wdgElement = ApplicationShell(wdgElement);

//      if ( wdgElement != NULL )
//      {
//  	Arg args[5];
//  	Cardinal argcount = 0;
//  	XtPointer pointer;
		
//  	XtSetArg(args[argcount],XmNuserData,&pointer); argcount++;
//  	XtGetValues(wdgElement,args,argcount);
//  	return ((xapplication *)pointer)->get_application_context();
//      }
//      else
//      {
//  	return NULL;
//      }
    return xapplication::get_application_context();
}

void CGuiElement::InternalDestroy()
{

}

void CGuiElement::PostCreate()
{

}

void CGuiElement::Manage()
{
    if (Container() != NULL && Created())
    {
	XtManageChild(Container());
    }
}

void CGuiElement::Unmanage()
{
    if ((Container() != NULL) && Created())
    {
	XtUnmanageChild(Container());
    }
}

void CGuiElement::Enable(bool bEnable)
{
    if ((Container() != NULL) && Created())
    {
	Arg args[2];
	Cardinal argcount = 0;

	XtSetArg(args[argcount], XmNsensitive, bEnable?True:False); argcount++;
	XtSetValues(Container(), args, argcount);
    }
}

void CGuiElement::Sync()
{
    XEvent theEvent;
    XtAppContext app = ApplicationContext();
	
    while(XtAppPending(app))
    {
	XtAppNextEvent(app,&theEvent);
	XtDispatchEvent(&theEvent);
    };
}
