/*******************************************************************************
 *
 * libgui
 *
 * libgui is a simple framework of GUI classes for X11/Motif widgets written in C++.
 *
 * The full text of the GNU General Public License can be found in the doc 
 * directory.
 *
 * Copyright (C) 2002, 2003 Stefan Hoefer
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * You can contact Stefan Hoefer via email at stefan@hoefer.ch
 *
 ******************************************************************************/

#ifndef GUIELEMENT_H

#define GUIELEMENT_H

#include <Xm/Xm.h>
#include <basic/idstring.h>
#include <basic/messageclient.h>

class CGuiElement : public CMessageClient
{
 private:
    Widget m_wdgParent;
    Widget m_wdgContainer;
    bool m_bCreated;
    bool m_bBeingDestroyed;
    bool m_bContainerDestroyed;
    unsigned long m_lStyle;
    //bool m_bWorkaroundXtDestroyWidget;
 private:
    inline void SetCreated(bool bCreated = true ) { m_bCreated = bCreated; }
    inline void SetBeingDestroyed(bool bBeingDestroyed = true) { m_bBeingDestroyed = bBeingDestroyed; }

    static void HandleDestroy(Widget w, XtPointer client_data, XmAnyCallbackStruct *call_data);

    inline void SetContainerDestroyed(bool bDestroyed=true) { m_bContainerDestroyed = bDestroyed; }
    inline bool ContainerDestroyed() const { return m_bContainerDestroyed; }
 protected:
    CGuiElement(Widget wdgParent, unsigned long lStyle=0L, int nId=0);
    CGuiElement(unsigned long lStyle = 0L, int nId=0);
		
    // STH sandalle SF virtual void InternalCreate(unsigned long lStyle) = 0;
    virtual void InternalCreate(unsigned long lStyle) = NULL;
    virtual void InternalDestroy();
    virtual void PostCreate();

    XtAppContext ApplicationContext();
    inline void SetStyle(unsigned long lStyle) { m_lStyle = lStyle; }
    //bool WorkaroundXtDestroyWidget();
    //void SetWorkaroundXtDestroyWidget();
    Widget ApplicationShell(Widget wdgParent);
 public:
    virtual ~CGuiElement();
    // STH sandalle SF virtual Widget Container() const = NULL;
    virtual Widget Container() const = 0;
    virtual void SetCaption(const char *strCaption);
    virtual const char *Caption();
    virtual void SetDimensions(int nX, int nY, int nWidth, int nHeight);
    virtual void Dimensions(int &nX, int &nY, int &nWidth, int &nHeight);
		
    void Create(unsigned long lStyle = 0L, Arg *pargs=NULL, Cardinal pargcount=0);
    void Create(Widget wdgParent, unsigned long lStyle = 0L, Arg *pargs=NULL, Cardinal pargcount=0);
    virtual void Destroy();
    inline Widget Parent() const { return m_wdgParent; }
    inline bool Created() const { return m_bCreated; }
    inline unsigned long Style() const { return m_lStyle; }
    inline bool BeingDestroyed() const { return m_bBeingDestroyed; }
    void Manage();
    void Unmanage();
    void Enable(bool bEnable=true);
    void Sync();
};

#endif

