/*******************************************************************************
 *
 * libgui Version 0.0.1
 *
 * libgui is a simple framework of GUI classes for X11/Motif widgets written in C++.
 *
 * The full text of the GNU General Public License can be found in the doc 
 * directory.
 *
 * Copyright (C) 2002, 2003 Stefan Hoefer
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * You can contact Stefan Hoefer via email at stefan@hoefer.ch
 *
 ******************************************************************************/

//#include "gui.h"
#include <X11/IntrinsicP.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Intrinsic.h>
#include <Xm/ScrolledW.h>
#include <Xm/TextF.h>
#include <X11/keysym.h>

#include "hierarchicaltreelist.h"
#include "coloring.h"
#include <basic/macrodefinitions.h>

#include <basic/cstrings.h>
#include <basic/globalstring.h>

/*!
*
* \fn CHierarchicalTreeBox::CHierarchicalTreeBox(Widget Parent, CTreeNode<CIdString> *pTree)
*
* \author STH
* \date 15.4.00
*
*/

CHierarchicalTreeBox::CHierarchicalTreeBox ( Widget Parent, unsigned long Style, int nId) 
    : graphical_list_box(Parent, nId)
{	
    SetStyle(Style);
}

void CHierarchicalTreeBox::FocusHandler(Widget w, XtPointer client_data, XEvent *call_data)
{
    XFocusChangeEvent *pEvent = (XFocusChangeEvent *)call_data;
 
    if (pEvent->type == FocusOut)
    {
	CHierarchicalTreeBox *pBox = (CHierarchicalTreeBox *)client_data;
	if ((pBox != NULL) && !pEvent->detail&NotifyNonlinear)
	{
	    pBox->OnLosingFocus();		
	}
    }
}

void CHierarchicalTreeBox::InternalCreate(unsigned long lStyle)
{
    graphical_list_box::InternalCreate(lStyle);

    XtAddEventHandler(Container(),FocusChangeMask,FALSE,(void (*)(_WidgetRec *, void *, _XEvent *, char *))FocusHandler,(XtPointer)this);

    m_nCurrentState = STATE_IDLE;
		
    SetMinWidth(100);
		
    SetCurrentSelectionNode((CGraphicalTreeNode *)NULL);

    m_wdgCurrentNewButton = NULL;
	
    m_pDisplay = XtDisplay(Parent());
	
    m_GC = XCreateGC(m_pDisplay, XtWindow(Parent()), 0, NULL);
	
    unsigned line_width = 1;
    int line_style = LineOnOffDash;
    int cap_style = CapButt;
    int join_style = JoinMiter;
	
    XSetLineAttributes(m_pDisplay, m_GC, line_width, line_style, cap_style, join_style);
	
    char dots[] = {1,1};
    XSetDashes(m_pDisplay, m_GC, 0, dots, 2);
	
    m_time = 0;
    m_wdgDoubleClick = NULL;
}

void CHierarchicalTreeBox::DeselectAll()
{
    SetCurrentSelectionNode(NULL);
    ReleaseMultiSelection();
}

/*!
*
* \fn void CHierarchicalTreeBox::Create(CTreeNode<CIdString> *pTree)
*
* \author STH
* \date 28.5.00
*
*/

void CHierarchicalTreeBox::Create(CTreeNode<CIdString> *pTree)
{
    DeselectAll();

    graphical_list_box::Create();

    int nFormerState = m_nCurrentState;
	
    m_nCurrentState = STATE_BUILDING;
	
    m_Tree = new CGraphicalTreeNode(*pTree);
	
    SetContents(m_Tree);

    Select((IDENT)0);
	
    XFlush(XtDisplay(Container()));
	
    m_nCurrentState = nFormerState;

    SetInputFocus();
}

void CHierarchicalTreeBox::OnLosingFocus()
{

}

/*!
*
* \fn void CHierarchicalTreeBox::RecursivelyAddNodes(CGraphicalTreeNode *pNode, graphical_list_item *pItem)
*
* \author STH
* \date 15.4.00
*
*/

void CHierarchicalTreeBox::RecursivelyAddNodes(CGraphicalTreeNode *pNode, graphical_list_item *pItem)
{
    Cardinal argcount = 0;
    Arg args[5];
	
    XtSetArg(args[argcount],XmNresizeWidth, FALSE); argcount++;
    XtSetArg(args[argcount],XmNresizeHeight, FALSE); argcount++;
	
    if (pItem != NULL)
    {
	XtSetValues(pItem->mysubform, args, argcount);
    }
    else
    {
	//XtSetValues(get_myform(), args, argcount);
    }
	
    int nIndex = 0;
	
    CGraphicalTreeNode *pSubNode;
	
    while ( pSubNode = (CGraphicalTreeNode *)pNode->Child(nIndex++) )
    {
	graphical_list_item *pSubItem;
		
	if ( pItem != NULL )
	{
	    pSubItem = pItem->add_item((const char *)pSubNode->Node());
	}
	else
	{
	    pSubItem = add_item((const char *)pSubNode->Node());
	}

	if ( Style() & HIERARCHICAL_LINES )
	{
	    XtAddCallback(pSubItem->Lines(), XmNexposeCallback, (XtCallbackProc)LinesCallback,(XtPointer)this);
	    XtAddCallback(pSubItem->Lines(), XmNresizeCallback, (XtCallbackProc)LinesCallback,(XtPointer)this);
	}
		
	pSubItem->SetData(pSubNode);
	pSubNode->SetItem(pSubItem);
		
	Arg args[5];
	Cardinal argcount = 0;
	Dimension width;
		
	XtSetArg(args[argcount],XmNwidth,&width); argcount++;
	XtGetValues(pSubItem->mylabel,args,argcount);
		
	argcount = 0;
		
	if ( width < MinWidth() )
	{
	    XtSetArg(args[argcount],XmNwidth,MinWidth()); argcount++;
	    XtSetArg(args[argcount],XmNalignment,XmALIGNMENT_BEGINNING); argcount++;
	}
				
	//XtSetArg(args[argcount],XmNmultiClick,XmMULTICLICK_KEEP); argcount++;
	XtSetValues(pSubItem->mylabel,args,argcount);
			
	XtAddCallback(pSubItem->mylabel,XmNactivateCallback,(XtCallbackProc)SelectCallback,(XtPointer)this);
						
	RecursivelyAddNodes(pSubNode,pSubItem);
    }
	
    if ( Style() & HIERARCHICAL_NEW_ITEM )
    {
	AddNewItemButton(pItem);
    }
	
    argcount = 0;
    XtSetArg(args[argcount],XmNresizeWidth, TRUE); argcount++;
    XtSetArg(args[argcount],XmNresizeHeight, TRUE); argcount++;
	
    if (pItem != NULL)
    {
	XtSetValues(pItem->mysubform, args, argcount);
	pItem->invisible();
    }
//      else
//      {
//      nIndex = 0;
//      while ( pSubNode = (CGraphicalTreeNode *)pNode->Child(nIndex++) )
//      {
//  	pSubNode->Item()->invisible();
//      }
//  	//XtSetValues(get_myform(), args, argcount);
//      }
}

void CHierarchicalTreeBox::DrawLines(graphical_list_item *pItem)
{
    Arg args[5];
    Cardinal argcount = 0;
    Dimension nWidth;
    Dimension nTotalHeight;
	
    if ( pItem->is_visible() )
    {
	Widget w = pItem->Lines();
	XtSetArg(args[argcount],XmNwidth, &nWidth); argcount++;
	XtSetArg(args[argcount],XmNheight, &nTotalHeight); argcount++;

	XtGetValues(w, args, argcount);
		
	Window win = XtWindow(w);
			
	XSetForeground(Disp(),m_GC,BackgroundColor());
	XFillRectangle(Disp(), win, m_GC,0,0,nWidth,nTotalHeight);
	XSetForeground(Disp(),m_GC,0L);
	
	int x = nWidth/2;
		
	int nNumber = pItem->card();
		
	if ( nNumber > 0 )
	{
	    int *pChildren = new int[pItem->card()];
			
	    GetChildPositions(pItem, pChildren);
			
	    int nHeight = 0;
	    int nMaxHeight =0;
			
	    for ( int nIndex = 0; nIndex < nNumber; nIndex++ )
	    {
		nHeight = pChildren[nIndex] + 5;
				
		if ( nHeight > nMaxHeight )
		{
		    nMaxHeight = nHeight;
		}
					
		XDrawLine(Disp(), win, m_GC, x, nHeight, nWidth, nHeight);		
	    }
			
	    XDrawLine(Disp(), win, m_GC, x, 0, x, nMaxHeight);		

	    delete pChildren;
	}
    }
}

void CHierarchicalTreeBox::DrawParentLines(graphical_list_item *pItem)
{
    Arg args[5];
    Cardinal argcount = 0;
    Dimension nWidth;
    Dimension nTotalHeight;
	
    if ( pItem->is_visible() )
    {
	while ( pItem = pItem->get_parent() )
	{
	    DrawLines(pItem);
	}
    }
}

void CHierarchicalTreeBox::LinesCallback(Widget w, XtPointer client_data, XmAnyCallbackStruct *callback_data)
{
    Arg args[5];
    Cardinal argcount = 0;
	
    CHierarchicalTreeBox *pBox = (CHierarchicalTreeBox *)client_data;
	
    switch(callback_data->reason )
    {
	case XmCR_EXPOSE:
	    if ( pBox->m_nCurrentState == STATE_IDLE )
	    {
		XtPointer pointer;
		XtSetArg(args[argcount], XmNuserData, &pointer); argcount++;
				
		XtGetValues(w, args, argcount);
			
		CGraphicalListItemContext *pContext = (CGraphicalListItemContext *)pointer;

		ASSERT(pContext->Type() == CGraphicalObjectContext::GRAPHICAL_LIST_ITEM);

		graphical_list_item *pItem = pContext->Object();
			
		pBox->DrawLines(pItem);				
	    }
	    break;
	case XmCR_RESIZE:
	    if ( pBox->m_nCurrentState == STATE_IDLE )
	    {
		XtPointer pointer;

		XtSetArg(args[argcount], XmNuserData, &pointer); argcount++;
				
		XtGetValues(w, args, argcount);
			
		CGraphicalListItemContext *pContext = (CGraphicalListItemContext *)pointer;

		ASSERT(pContext->Type() == CGraphicalObjectContext::GRAPHICAL_LIST_ITEM);

		graphical_list_item *pItem = pContext->Object();
			
		pBox->DrawParentLines(pItem);				
	    }
	    break;
    }
}

void CHierarchicalTreeBox::GetChildPositions(graphical_list_item *pItem, int *pChildren)
{
    Arg args[5];
    Cardinal argcount = 0;
    Dimension y;
	
    XtSetArg(args[argcount],XmNy, &y); argcount++;
	
    for ( int nIndex = 0; nIndex < pItem->card(); nIndex++ )
    {
	graphical_list_item *pSubItem = (*pItem)[nIndex];
		
	XtGetValues(pSubItem->myform,args, argcount);
		
	pChildren[nIndex] = y;
    }
}

/*void CHierarchicalTreeBox::HandleSpecialInput(CGraphicalTreeNode *pNode)
{

}*/

void CHierarchicalTreeBox::AddNewItemButton(graphical_list_item *pItem)
{
    graphical_list_item *pSubItem;

    if ( pItem != NULL )
    {
	pSubItem = pItem->add_item(" ");
    }
    else{
	pSubItem = add_item(" ");
    }
	
    XtAddCallback(pSubItem->mylabel,XmNactivateCallback,(XtCallbackProc)NewCallback,(XtPointer)this);
					
    Arg args[5];
    Cardinal argcount;
	
    argcount = 0;
	
    color_coding color(XtDisplay(pSubItem->mylabel));
    XtSetArg(args[argcount],XmNbackground,color.get_rgb(.9,.7,.7)); argcount++;
	
    XtSetValues(pSubItem->mylabel,args,argcount);

    // make sure that the button has a certain minimal width
	
    argcount = 0;
    Dimension width;
    XtSetArg(args[argcount],XmNwidth,&width); argcount++;
    XtGetValues(pSubItem->mylabel,args,argcount);

    if ( width < MinWidth() )
    {
	argcount = 0;
	XtSetArg(args[argcount],XmNwidth,MinWidth()); argcount++;
	XtSetArg(args[argcount],XmNalignment,XmALIGNMENT_BEGINNING); argcount++;
	XtSetValues(pSubItem->mylabel,args,argcount);
    }
}

void CHierarchicalTreeBox::AddNewNode(CGraphicalTreeNode *pTree)
{
    if (pTree != NULL)
    {
	CGraphicalTreeNode *pNewNode = new CGraphicalTreeNode(*pTree);

	InsertNode(pNewNode);

	graphical_list_item *pItem = pNewNode->Item();
	
	do
	{
	    DrawLines(pItem);
	} while ( pItem = pItem->get_parent() );
	
	// Select the newly added item, but do not notify the user of it
	
	Select(pNewNode,true,false,false);
	
	// now tell the user that a new item has been added
	
	NewItemAdded(pNewNode);

	StartInput();
    }
}

void CHierarchicalTreeBox::SetContents(CGraphicalTreeNode *pTree)
{
    DeleteAllItems();
    DeselectAll();

    //int nIndex = 0;
	
    //CGraphicalTreeNode *pNode;
	
    /*while ( pNode = (CGraphicalTreeNode *)pTree->Child(nIndex++) )
      {
      graphical_list_item *pItem = add_item((char *)pNode->Node());
      pItem->SetData(pNode);
      pNode->SetItem(pItem);
		
      Arg args[5];
      Cardinal argcount = 0;
		
      Dimension width;
		
      XtSetArg(args[argcount],XmNwidth,&width); argcount++;
      XtGetValues(pItem->mylabel,args,argcount);
      argcount = 0;
      if ( width < MinWidth() )
      {
      XtSetArg(args[argcount],XmNwidth,MinWidth()); argcount++;
      XtSetArg(args[argcount],XmNalignment,XmALIGNMENT_BEGINNING); argcount++;
      }

      XtSetArg(args[argcount],XmNmultiClick,XmMULTICLICK_KEEP); argcount++;
      XtSetValues(pItem->mylabel,args,argcount);
			
      XtAddCallback(pItem->mylabel,XmNactivateCallback,SelectCallback,(XtPointer)this);
				
      RecursivelyAddNodes(pNode);
      }*/
	
    RecursivelyAddNodes(pTree);
	
    /*if ( m_Style & HIERARCHICAL_NEW_ITEM ){
      AddNewItemButton((graphical_list_item *)NULL);
      }*/
}

void CHierarchicalTreeBox::SetContents(CTreeNode<CIdString> *pTree)
{
    m_CurrentSelection = NULL;
    delete m_Tree;
	
    m_Tree = new CGraphicalTreeNode(*pTree);
	
    SetContents(m_Tree);
}

void CHierarchicalTreeBox::SetCurrentSelection(CIdString &strData)
{
    if ( m_CurrentSelection != NULL )
    {
	XtPointer data;
	Arg args[5];
	Cardinal argcount;
		
	//argcount = 0;
	//XtSetArg(args[argcount],XmNuserData,&data); argcount++;
		
	//XtGetValues(m_CurrentSelection,args,argcount);
		
	graphical_list_item *pItem = m_CurrentSelection->Item();

	char *strBuffer = new char[strData.Length()+1];
	strcpy(strBuffer,(const char *)strData);
		
	XmString string = XmStringCreate(strBuffer,XmFONTLIST_DEFAULT_TAG);
	argcount = 0;
	delete strBuffer;
		
	//XtSetArg(args[argcount],XmNuserData,strData.Id()); argcount++;
	XtSetArg(args[argcount],XmNlabelString,string); argcount++;
		
	XtSetValues(pItem->mylabel,args,argcount);
		
	//XmStringFree(string);	
		
	CGraphicalTreeNode *pNode = (CGraphicalTreeNode *)pItem->Data();
		
	if ( pNode != NULL )
	{
	    pNode->Node() = strData;
	}
    }
}

void CHierarchicalTreeBox::DeleteCurrentSelection()
{
    /*if ( m_CurrentSelection != NULL ){
	XtPointer data;
	Arg args[5];
	Cardinal argcount;
		
	//argcount = 0;
	//XtSetArg(args[argcount],XmNuserData,&data); argcount++;
		
	//XtGetValues(m_CurrentSelectionButton,args,argcount);
		
	graphical_list_item *pItem = m_CurrentSelection->Item();
		
	pItem->RemoveFromList();
		
	CGraphicalTreeNode *pNode = (CGraphicalTreeNode *)pItem->Data();
	if ( pNode != NULL )
	{
	    // Delete it from the tree...
			
	    delete pNode;
	}
		
	m_CurrentSelection = NULL;
	}*/

    DeleteNode(m_CurrentSelection);
}

void CHierarchicalTreeBox::DeleteNode(CGraphicalTreeNode *pNode)
{
    if (pNode != NULL)
    {
	CIdString strItem = pNode->Node();

	graphical_list_item *pItem = pNode->Item();
		
	pItem->RemoveFromList();

	m_CutList -= pNode;
	m_CopyList -= pNode;
	m_MultiSelection -= pNode;
	
	if ( pNode != NULL )
	{
	    // Delete it from the tree...
			
	    delete pNode;
	}

	if (pNode == m_CurrentSelection)
	{
	    m_CurrentSelection = NULL;
	}

	//Remove the deleted node from the copy and the cut buffer

	m_CopyList -= pNode;
	m_CutList -= pNode;

	ItemDeleted(strItem);
    }
}

CIdString CHierarchicalTreeBox::CurrentSelection()
{
    CIdString strData;

    if ( m_CurrentSelection != NULL ){
	strData = m_CurrentSelection->Node();
    }
	
    return strData;
}

void CHierarchicalTreeBox::SelectCallback(Widget w, XtPointer client_data, XmPushButtonCallbackStruct *callback_data)
{
    CHierarchicalTreeBox *pTreeBox = (CHierarchicalTreeBox *)client_data;

    bool bDoubleClick = false;
    bool bShift = false;

    if(callback_data->event->type == ButtonRelease)
    {
	XButtonEvent *pEvent = (XButtonEvent *)callback_data->event;
		
	Time time = pEvent->time;
		
	if (w == pTreeBox->m_wdgDoubleClick)
	{
	    if (time - pTreeBox->m_time <= 200)
	    {
		bDoubleClick = true;
	    }
			
	    pTreeBox->m_time = time;
	}
	else
	{
	    pTreeBox->m_wdgDoubleClick = w;
			
	    pTreeBox->m_time = time;
	}

	bShift = pEvent->state&ShiftMask;
    }

    Arg args[5];
    Cardinal argcount = 0;

    XtPointer pointer;

    XtSetArg(args[argcount],XmNuserData,&pointer); argcount++;
    XtGetValues(w,args,argcount);
	
    CGraphicalListItemContext *pContext = (CGraphicalListItemContext *)pointer;
    ASSERT(pContext->Type() == CGraphicalObjectContext::GRAPHICAL_LIST_ITEM);

    graphical_list_item *pItem = pContext->Object();

    //XmProcessTraversal(pTreeBox->Container(),XmTRAVERSE_CURRENT);
    pTreeBox->SetInputFocus();

    if ( pItem != NULL )
    {	
	bool bAendern = ((pTreeBox->m_CurrentSelection != NULL) && (pTreeBox->m_CurrentSelection == (CGraphicalTreeNode *)pItem->Data()) );
		
	// Do not notify the user yet that the selection has changed! (Or otherwise he won't be able to react by deleting the treebox)
		
	CGraphicalTreeNode *pNode = (CGraphicalTreeNode *)pItem->Data();

	if (pTreeBox->Style()&HIERARCHICAL_MULTI_SELECT)
	{
	    if (pTreeBox->InMultiSelection(pNode) && bShift)
	    {
		if (pTreeBox->CurrentSelectionNode() == pNode)
		{
		    pTreeBox->Select(pNode, false, bShift, false);
		}
		pTreeBox->RemoveFromMultiSelection(pNode);
		pTreeBox->ColorNode(pNode);
	    }
	    else
	    {
		pTreeBox->Select(pNode, true, bShift, false);
	    }
	}
	else
	{
	    pTreeBox->Select(pNode, true, bShift, false);
	}

	//pTreeBox->m_CurrentSelection = (CGraphicalTreeNode *)pItem->Data();
		
	/*if ( ((CGraphicalTreeNode *)pItem->Data())->SpecialFlag() )
	{
	    pTreeBox->HandleSpecialInput((CGraphicalTreeNode *)pItem->Data());
	}
	else*/ 
	if (!(pTreeBox->Style() & HIERARCHICAL_READONLY))
	{
	    /*if ( callback_data->reason == XmCR_ACTIVATE)
	      {
	      puts("Ok");
	      }
			
	      printf("Es wurde %dmal geklickt\n",callback_data->click_count);
	      argcount = 0;
	      unsigned char c;
	      XtSetArg(args[argcount],XmNmultiClick,&c); argcount++;
	      XtGetValues(pItem->mylabel,args,argcount);
			
	      if ( c == XmMULTICLICK_KEEP )
	      {
	      puts("Multiclick ist aktiviert");
	      }
	      else
	      {
	      puts("Mist!");
	      }

	      printf("Die Multiclick Zeit betraegt %d Millisekunden\n",XtGetMultiClickTime(XtDisplay(w)));
	    */
			
	    if ( /*callback_data->click_count > 1*/ bAendern )
	    {
		pTreeBox->StartInput();
	    }
	    else
	    {	
		pTreeBox->SelectionChanged();
	    }
	}
	else
	{
	    // Now tell the user that something's changed
			
	    pTreeBox->SelectionChanged();
			
	    if (bDoubleClick)
	    {
		pTreeBox->OnOk();
	    }
	    else
	    {
		pTreeBox->SetInputFocus();
	    }
	}
    }	
}

void CHierarchicalTreeBox::NewCallback(Widget w, XtPointer client_data, XmAnyCallbackStruct *callback_data)
{
    CHierarchicalTreeBox *pTreeBox = (CHierarchicalTreeBox *)client_data;

    XtPointer data;
    Arg args[5];
    Cardinal argcount=0;

    XtSetArg(args[argcount],XmNuserData,&data); argcount++;
	
    XtGetValues(w,args,argcount);
    CGraphicalListItemContext *pContext = (CGraphicalListItemContext *)data;
    ASSERT(pContext->Type() == CGraphicalObjectContext::GRAPHICAL_LIST_ITEM);

    graphical_list_item *pItem = pContext->Object();
    /*CGraphicalTreeNode *pNewNode = NULL;

      if ( pItem->get_parent() != NULL )
      {
      CGraphicalTreeNode *pNode = (CGraphicalTreeNode *)pItem->get_parent()->Data();
	
      if ( pNode != NULL )
      {
      pNewNode = new CGraphicalTreeNode(*pNode);
      pNewNode->Node() = "";
      }
      }
      else
      {
      pNewNode = new CGraphicalTreeNode(*pTreeBox->m_Tree);
      pNewNode->Node() = "";
      }
	
      pTreeBox->m_Tree->GetHideFlag();
      pTreeBox->SetContents(pTreeBox->m_Tree);
      pTreeBox->m_Tree->SetHideFlag();
      pTreeBox->m_CurrentSelection = pNewNode;*/
	
    pTreeBox->m_wdgCurrentNewButton = pItem;
    pTreeBox->StartInput();
}

void CHierarchicalTreeBox::SelectionChanged()
{
    SendMessage(EVENT_MASK_SELECTION_CHANGED);
}

CHierarchicalTreeBox::~CHierarchicalTreeBox()
{
    //XtDestroyWidget(ScrolledWindow());
	
    delete m_Tree;
	
    XFreeGC(XtDisplay(Container()), m_GC);
}

void CGraphicalTreeNode::operator=(CTreeNode<CIdString> &Tree)
{
    Node() = Tree.Node();
	
    int nIndex = 0;
    CGraphicalTreeNode *pSubNode;
	
    while ( CTreeNode<CIdString> *pNode = Tree.Child(nIndex) )
    {
	pSubNode = new CGraphicalTreeNode(*this);
	*pSubNode = *pNode;
		
	nIndex++;
    }
}

CGraphicalTreeNode::CGraphicalTreeNode(CTreeNode<CIdString> &Tree)
    : CTreeNode<CIdString>()
{
    CommonConstruct();
	
    *this = Tree;
    SetHideSub(true);
}

CGraphicalTreeNode::CGraphicalTreeNode(CGraphicalTreeNode &pNode)
    : CTreeNode<CIdString>((CTreeNode<CIdString> *)&pNode)
{
    CommonConstruct();
	
    SetHideSub(true);
}

CGraphicalTreeNode::CGraphicalTreeNode()
{
    CommonConstruct();
	
    SetHideSub(true);
}

void CGraphicalTreeNode::Check(bool bSet)
{
    graphical_list_item *pItem = Item();
    
    if (pItem != NULL)
    {
	Item()->Check(bSet);
    }

    m_bChecked = bSet;
}

void CGraphicalTreeNode::CommonConstruct()
{
    //SetSpecialFlag(false);
    m_pItem = NULL;
    m_bChecked = false;
}

void CGraphicalTreeNode::GetHideFlag()
{
    if ( Item() != NULL )
    {
	SetHideSub(!Item()->is_visible());
    }
	
    int nIndex = 0;
	
    while ( CGraphicalTreeNode *pNode = (CGraphicalTreeNode *)Child(nIndex) )
    {
	pNode->GetHideFlag();
	nIndex++;
    }
}

void CGraphicalTreeNode::SetHideFlag()
{
    if ( Item() != NULL )
    {
	if ( HideSub() )
	{
	    Item()->invisible();
	}
	else
	{
	    Item()->visible();
	}
    }
	
    int nIndex = 0;
	
    while ( CGraphicalTreeNode *pNode = (CGraphicalTreeNode *) Child(nIndex) )
    {
	pNode->SetHideFlag();
	nIndex++;
    }
}

/*!
*
* \fn void CHierarchicalTreeBox::StartInput()
*
* \author STH
* \date 11.5.00
*
*/

void CHierarchicalTreeBox::StartInput()
{
    //if ( m_CurrentSelection != NULL )
    if ( m_wdgCurrentNewButton != NULL || m_CurrentSelection != NULL)
    {
	if ( m_nCurrentState == STATE_INPUT )
	{
	    EndInput();
	}
		
	m_nCurrentState = STATE_INPUT;
	
	//graphical_list_item *pItem = m_CurrentSelection->Item();
		
	graphical_list_item *pItem = m_wdgCurrentNewButton != NULL ? m_wdgCurrentNewButton : m_CurrentSelection->Item();
		
	Arg args[20];
	Cardinal argcount = 0;
	Dimension width, height;
	Dimension x,y;
		
	XtSetArg(args[argcount],XmNwidth, &width); argcount++;
	XtSetArg(args[argcount],XmNheight,&height); argcount++;
	//XtSetArg(args[argcount],XmNx,&x); argcount++;
	//XtSetArg(args[argcount],XmNy,&y); argcount++;
		
	XtGetValues(pItem->mylabel,args,argcount);
		
	if ( width < 100 )
	{
	    width = 100;
	}
	if ( height < 30 )
	{
	    height = 30;
	}
		
	ComputeCoordinates(pItem->mylabel,Container(),x,y);
	//ComputeCoordinates(pItem->mylabel,XtParent(pItem->myform),x,y);
	//ComputeCoordinates(pItem->mylabel,ScrolledWindow(),x,y);
		
	argcount = 0;
	XtSetArg(args[argcount],XmNwidth,width); argcount++;
	XtSetArg(args[argcount],XmNheight,height); argcount++;
	XtSetArg(args[argcount],XmNx,x); argcount++;
	XtSetArg(args[argcount],XmNy,y); argcount++;
		
	if ( m_wdgCurrentNewButton == NULL )
	{
	    XtSetArg(args[argcount],XmNvalue,(const char *)m_CurrentSelection->Node()); argcount++;
	}
		
	//m_TextInput = XmCreateTextField(pItem->myform,"textinput",args,argcount);
	m_TextInput = XmCreateTextField(Container(),"textinput",args,argcount);
	//m_TextInput = XmCreateTextField(ScrolledWindow(),"textinput",args,argcount);
		
	XtManageChild(m_TextInput);

	XtAddCallback(m_TextInput,XmNactivateCallback,(XtCallbackProc)ValidateInput,(XtPointer)this);

	//XmProcessTraversal(m_TextInput,XmTRAVERSE_CURRENT);
	SetInputFocus();
    }
}

void CHierarchicalTreeBox::EndInput()
{
    if ( m_nCurrentState == STATE_INPUT )
    {
	bool bNewItemAdded = false;
	CGraphicalTreeNode *pActualNode = m_CurrentSelection;
		
	if ( m_wdgCurrentNewButton != NULL )
	{
	    CGraphicalTreeNode *pNewNode = NULL;

	    if ( m_wdgCurrentNewButton->get_parent() != NULL )
	    {
		pNewNode = new CGraphicalTreeNode(*(CGraphicalTreeNode *)m_wdgCurrentNewButton->get_parent()->Data());
		pNewNode->Node() = "";
	    }
	    else
	    {
		pNewNode = new CGraphicalTreeNode(*m_Tree);
		pNewNode->Node() = "";
	    }
			
	    //m_Tree->GetHideFlag();
	    //SetContents(m_Tree);
	    InsertNode(pNewNode);
	    //m_Tree->SetHideFlag();
	    //m_CurrentSelection = pNewNode;
			
	    bNewItemAdded = true;
	    pActualNode = pNewNode;
	}
		
	//-------
	
	Arg args[5];
	Cardinal argcount = 0;
	char *pText;
		
	XtSetArg(args[argcount],XmNvalue,&pText); argcount++;
		
	XtGetValues(m_TextInput,args,argcount); 
		
	pActualNode->Node() = pText;
		
	XmString string = XmStringCreate(pText,XmFONTLIST_DEFAULT_TAG);
		
	argcount = 0;
	XtSetArg(args[argcount],XmNlabelString,string); argcount++;
	XtSetValues(pActualNode->Item()->mylabel,args,argcount);
		
	//XmStringFree(string); 
	XtFree(pText);

	// Inform the user that the text has changed
		
	if ( bNewItemAdded )
	{
	    ASSERT(pActualNode != NULL);
			
	    graphical_list_item *pItem = pActualNode->Item();
			
	    do
	    {
		DrawLines(pItem);
	    } while ( pItem = pItem->get_parent() );
			
	    // Select the newly added item, but do not notify the user of it
			
	    Select(pActualNode,true,false,false);
			
	    // now tell the user that a new item has been added
			
	    NewItemAdded(pActualNode);
	}
	else
	{
	    TextChanged();
	}
			
	CancelInput();
    }
	
    m_wdgCurrentNewButton = NULL;
}

void CHierarchicalTreeBox::CancelInput()
{
    if ( m_nCurrentState == STATE_INPUT )
    {
	XtDestroyWidget(m_TextInput);
	m_nCurrentState = STATE_IDLE;
    }	
}

void CHierarchicalTreeBox::ValidateInput(Widget w, XtPointer client_data, XmAnyCallbackStruct *callback_data)
{
    CHierarchicalTreeBox *pTreeBox = (CHierarchicalTreeBox *)client_data;
	
    pTreeBox->EndInput();
}

void CHierarchicalTreeBox::TextChanged()
{
    SendMessage(EVENT_MASK_VALUE_UPDATE);
}

/*void CHierarchicalTreeBox::DAInput(Widget w, XtPointer client_data, XmDrawingAreaCallbackStruct *callback_data)
{
	puts("1");
}*/

void CHierarchicalTreeBox::ComputeCoordinates(Widget Current, Widget Frame, Dimension &x, Dimension &y)
{
    Dimension dx,dy;
    Arg args[5];
    Cardinal argcount;
		
    x = 0;
    y = 0;
		
    do
    {
	argcount = 0;
	XtSetArg(args[argcount],XmNx,&dx); argcount++;
	XtSetArg(args[argcount],XmNy,&dy); argcount++;
		
	XtGetValues(Current,args,argcount);
	x += dx; 
	y += dy;
			
	Current = XtParent(Current);
    }while(Current != Frame);
}

bool CHierarchicalTreeBox::NotifyScroll()
{
    EndInput();

    return false;
}

bool CHierarchicalTreeBox::NotifyInput(XmDrawingAreaCallbackStruct *call_data)
{
    bool bResult = true;

    XKeyEvent *KeyEvent = (XKeyEvent *)call_data->event;
	
    if ( KeyEvent->type == KeyRelease )
    {
	CGraphicalTreeNode *pNextNode;
		
	switch ( XLookupKeysym(KeyEvent,0) )
	{
	    case XK_Left:
		if (CurrentSelectionNode())
		{
		    pNextNode = CurrentSelectionNode()->Parent();
	    	
		    if ( pNextNode != NULL && pNextNode->Parent() != NULL)
		    {
			Select(pNextNode);
		    }
		}
	    	break;
	    case XK_Right:
		if (CurrentSelectionNode())
		{
		    graphical_list_item *pItem = CurrentSelectionNode()->Item();	

		    if ( pItem->is_visible() && pItem->is_visible_on_screen() )
		    { 
			pNextNode = CurrentSelectionNode()->Child(0);
			    	
			if ( pNextNode != NULL )
			{
			    Select(pNextNode);
			}
		    }
		}
		break;
	    case XK_Up:
		if (CurrentSelectionNode())
		{
		    pNextNode = CurrentSelectionNode()->Pred();
	    	
		    if ( pNextNode != NULL )
		    {
			Select(pNextNode);
		    }
		}
	    	break;
	    case XK_Down:
		if (CurrentSelectionNode())
		{
		    pNextNode = CurrentSelectionNode()->Succ();
	    	
		    if ( pNextNode != NULL )
		    {
			Select(pNextNode);
		    }
		}
	    	break;
	    case XK_Escape:
	    	OnCancel();
		bResult = false;
	    	break;
	    case XK_Return:
		if (CurrentSelectionNode())
		{
		    /*if (CurrentSelectionNode()->SpecialFlag())
		      {
		      HandleSpecialInput(CurrentSelectionNode());
		      }
		      else
		      {*/
		    OnOk();
		    /*}*/
		    bResult = false;
		}
	    	break;
	    case XK_space:
		if (CurrentSelectionNode())
		{
		    if ( CurrentSelectionNode()->Item()->is_visible() )
		    {
			CurrentSelectionNode()->Item()->invisible();
		    }
		    else
		    {
			CurrentSelectionNode()->Item()->visible();
		    }
		}
	    	break;
	}
    }
    /*else if (KeyEvent->type == KeyPress)
      {
      puts("Received keypress!");

      switch (XLookupKeysym(KeyEvent,0))
      {
      case XK_Tab:
      OnTab();
      bResult = false;
      break;
      }
      }*/

    return bResult;
}

void CHierarchicalTreeBox::OnTab()
{

}

void CHierarchicalTreeBox::Select(CGraphicalTreeNode *pNode, bool bSelect, bool bShift, bool bNotifyUser)
{
    if (pNode != NULL)
    {
	if (pNode != CurrentSelectionNode())
	{
	    if (bSelect)
	    {
		if (!bShift)
		{
		    ReleaseMultiSelection();
		}

		Select(CurrentSelectionNode(), false, bShift);
			
		/*Widget wdgButton = pNode->Item()->mylabel;
						
		Arg args[5];
		Cardinal argcount=0;
		color_coding color(XtDisplay(wdgButton));
				
		XtSetArg(args[argcount],XmNbackground,color.get_rgb(0.2,.2,1.)); argcount++;
		XtSetValues(wdgButton,args,argcount);*/
					
//  		if (bSelect)
//  		{
//  		    OpenNode(pNode);
//  		}

		if ( !Visible(pNode) )
		{
		    CenterNode(pNode);
		}
				
		SetCurrentSelectionNode(pNode);
		
		ColorNode(pNode);

		if ( bNotifyUser && m_nCurrentState != STATE_BUILDING)
		{
		    SelectionChanged();
		}
	    }
	}
	else
	{
	    if (!bSelect)
	    {
		/*Widget wdgButton = CurrentSelectionNode()->Item()->mylabel;
				
		Arg args[5];
		Cardinal argcount=0;
		color_coding color(XtDisplay(wdgButton));
				
				//XtSetArg(args[argcount],XmNbackground,color.get_rgb(0.75,0.75,0.75)); argcount++;
		XtSetArg(args[argcount],XmNbackground,BackgroundColor()); argcount++;
		XtSetValues(wdgButton,args,argcount);*/

		SetCurrentSelectionNode(NULL);
		if (!bShift)
		{
		    RemoveFromMultiSelection(pNode);
		}

		ColorNode(pNode);
	    }
	}
    }
    else
    {
	if (bSelect)
	{
	    pNode = CurrentSelectionNode();

	    Select(CurrentSelectionNode(),false, bShift);
	}
    } 
}

void CHierarchicalTreeBox::Select(IDENT idNumber, IDENT idType)
{
    CGraphicalTreeNode *pTree;
	
    if ( idNumber == 0 )
    {
	pTree = Tree()->Child(0);
    }
    else
    {
	pTree = Tree()->GetById(idNumber, idType);				
    }
		
    Select(pTree);
}

CGraphicalTreeNode *CGraphicalTreeNode::GetById(IDENT idNumber, IDENT idType)
{
    if (Id() == idNumber && (idType == 0 || Type() == idType))
    {
	return this;
    }
    else
    {
	CGraphicalTreeNode *pTree, *pRetTree;
		
	for (int nIndex = 0; (pTree = Child(nIndex)) != NULL ; nIndex++)
	{
	    pRetTree = pTree->GetById(idNumber, idType);
			
	    if (pRetTree != NULL)
	    {
		return pRetTree;
	    }
	}
		
	ASSERT(pTree == NULL);
		
	return NULL;
    }
}

void CHierarchicalTreeBox::OnCancel()
{

}

void CHierarchicalTreeBox::OnOk()
{

}

CGraphicalTreeNode *CGraphicalTreeNode::Succ()
{
    if ( Parent() != NULL )
    {
	CGraphicalTreeNode *pNode;
		
	int nIndex = 0;
		
	do
	{
	    pNode = Parent()->Child(nIndex++);
			
	    ASSERT(pNode != NULL );
	}while ( pNode != this );
		
	ASSERT(pNode == this);
		
	return Parent()->Child(nIndex);
    }
    else
    {
	return NULL;
    }
}

CGraphicalTreeNode *CGraphicalTreeNode::Pred()
{
    if ( Parent() != NULL )
    {
	CGraphicalTreeNode *pNode;
		
	int nIndex = 0;
		
	do
	{
	    pNode = Parent()->Child(nIndex++);
			
	    ASSERT(pNode != NULL );
	}while ( pNode != this );
		
	ASSERT(pNode == this);
		
	if ( nIndex >= 2 )
	{
	    return Parent()->Child(nIndex-2);
	}
	else
	{
	    return NULL;
	}
    }
    else
    {
	return NULL;
    }
}

bool CHierarchicalTreeBox::Visible(CGraphicalTreeNode *pNode )
{
    graphical_list_item *pItem = pNode->Item();
    switch ( XmGetVisibility(pItem->mylabel) )
    {
	case XmVISIBILITY_PARTIALLY_OBSCURED:
	case XmVISIBILITY_FULLY_OBSCURED:
	    return false;
	    break;
	default:
	    return true;
	    break;
    }
}

void CHierarchicalTreeBox::CenterNode(CGraphicalTreeNode *pNode )
{
    //puts("I'll have to center this widget");
	
    XmScrollVisible(ScrolledWindow(),pNode->Item()->mylabel,10,10);
}

unsigned long CHierarchicalTreeBox::BackgroundColor()
{
    Arg args[6];
    Cardinal argcount = 0;
    unsigned long lColor;
	
    XtSetArg(args[argcount],XmNbackground, &lColor); argcount++;
    XtGetValues(ScrolledWindow(),args,argcount);
	
    return lColor;
}

void CHierarchicalTreeBox::NewItemAdded(CGraphicalTreeNode *pNode)
{
    SendMessage(EVENT_MASK_NEW_ADDED);
}

void CHierarchicalTreeBox::ItemDeleted(const CIdString &strItem)
{

}

void CHierarchicalTreeBox::InsertNode(CGraphicalTreeNode *pNode)
{
    CGraphicalTreeNode *pParentNode = pNode->Parent();
	
    ASSERT(pParentNode != NULL);
    ASSERT(pNode->Item() == NULL);
    Arg args[5];
    Cardinal argcount = 0;
    int nChildren;
	
    XtSetArg(args[argcount],XmNnumChildren,&nChildren); argcount++;
	
    if ( pParentNode->Item() != NULL )
    {
	XtGetValues(pParentNode->Item()->get_mysubform(),args,argcount);
    }
    else
    {
	XtGetValues(get_myform(),args,argcount);	
    }
	
    // If this treebox has an "empty" new button, insert the new item at the second last place
	
    if (Style() & HIERARCHICAL_NEW_ITEM)
    {
	nChildren--;
    }
	
    graphical_list_item *pItem = NULL;
	
    if ( pParentNode->Item() != NULL )
    {
	pItem = pParentNode->Item()->add_item((const char *)pNode->Node(),nChildren);
    }
    else
    {
	pItem = add_item((const char *)pNode->Node(),nChildren);	
    }
	
    pItem->SetData(pNode);
    pNode->SetItem(pItem);	

    XtAddCallback(pItem->mylabel,XmNactivateCallback,(XtCallbackProc)SelectCallback,(XtPointer)this);
	
    if (Style() & HIERARCHICAL_NEW_ITEM)
    {
	AddNewItemButton(pItem);
    }	
}

bool CHierarchicalTreeBox::HasContextMenu()
{
    return (Style() & HIERARCHICAL_STANDARD_POPUP);
}

void CHierarchicalTreeBox::CreateContextMenu(CSimpleObjectList<CIdString> &Items)
{
    if (Style() & HIERARCHICAL_STANDARD_POPUP)
    {
	CreateInsertContextMenu(Items, false);
	CIdString String = "";
	String.SetId(0);
	Items += String;
	CreateEditContextMenu(Items, false);
    }
}

void CHierarchicalTreeBox::CreateInsertContextMenu(CSimpleObjectList<CIdString> &Items, bool bTop)
{
    CIdString String;
    String = LANGUAGE_STRING(IDS_INSERT);
    String.SetId(ID_POPUP_INSERT);
    Items += String;

    if (!bTop)
    {
	String = LANGUAGE_STRING(IDS_DELETE);
	String.SetId(ID_POPUP_DELETE);
	Items += String;
    }
}

void CHierarchicalTreeBox::CreateEditContextMenu(CSimpleObjectList<CIdString> &Items, bool bTop)
{
    CIdString String;
    if (!bTop)
    {
	String = LANGUAGE_STRING(IDS_COPY);
	String.SetId(ID_POPUP_COPY);
	Items += String;
	String = LANGUAGE_STRING(IDS_CUT);
	String.SetId(ID_POPUP_CUT);
	Items += String;
    }
    String = LANGUAGE_STRING(IDS_PASTE);
    String.SetId(ID_POPUP_PASTE);
    Items += String;
    String = LANGUAGE_STRING(IDS_LINK);
    String.SetId(ID_POPUP_LINK);
    Items += String;
}

void CHierarchicalTreeBox::CreateTopContextMenu(CSimpleObjectList<CIdString> &Items)
{
    if (Style() & HIERARCHICAL_STANDARD_POPUP)
    {
	CreateInsertContextMenu(Items, true);
	CIdString String = "";
	String.SetId(0);
	Items += String;
	CreateEditContextMenu(Items, true);
    }
}

/*void CHierarchicalTreeBox::OnMapPopup(void *pLauncher)
{
    graphical_list_item *pItem = (graphical_list_item *)pLauncher;
    CGraphicalTreeNode *pNode = (CGraphicalTreeNode *)pItem->Data();
    Select(pNode, true, false);
}*/

void CHierarchicalTreeBox::OpenNode(CGraphicalTreeNode *pNode)
{
    CGraphicalTreeNode *pActualNode = pNode;

    while (pNode != NULL)
    {
	graphical_list_item *pItem = pNode->Item();

	if (pItem != NULL)
	{
	    if (!pItem->is_visible())
	    {
		pItem->visible();
	    }
	}
	pNode = pNode->Parent();
    }

    OnOpenNode(pActualNode);
}

void CHierarchicalTreeBox::OnOpenNode(CGraphicalTreeNode *pNode)
{
    SendMessage(EVENT_MASK_VALUE_UPDATE, (void *)pNode);
}

void CHierarchicalTreeBox::OnPreparePopup(Widget wdgLauncher, bool bShift, CPopupMenuHandle *pHandle)
{
    XtPointer pointer;

    Arg args[5];
    Cardinal argcount = 0;

    XtSetArg(args[argcount], XmNuserData, &pointer); argcount++;
    XtGetValues(wdgLauncher, args, argcount);

    CGraphicalObjectContext *pContext = (CGraphicalObjectContext *)pointer;

    if (pContext != NULL)
    {
	switch (pContext->Type())
	{
	    case CGraphicalObjectContext::GRAPHICAL_LIST_ITEM:
	    {
		CGraphicalListItemContext *pItemContext = (CGraphicalListItemContext *)pContext;

		graphical_list_item *pItem = pItemContext->Object();
		CGraphicalTreeNode *pNode = (CGraphicalTreeNode *)pItem->Data();

		Select(pNode, true, bShift, false);		
	    }
	    break;
	}
    }
    
    pHandle->EnableItem(ID_POPUP_PASTE, !((m_CopyList.card() == 0) && (m_CutList.card() == 0)));
    pHandle->EnableItem(ID_POPUP_COPY, !((m_MultiSelection.card() == 0) && (CurrentSelectionNode() == NULL)));
    pHandle->EnableItem(ID_POPUP_CUT, !((m_MultiSelection.card() == 0) && (CurrentSelectionNode() == NULL)));
    pHandle->EnableItem(ID_POPUP_LINK, m_CopyList.card() > 0);
}

void CHierarchicalTreeBox::LinkItems(CGraphicalTreeNode *pNode)
{
    //STHTODO
}

void CHierarchicalTreeBox::OnPopupMenu(int nMenuId, int nId)
{
    switch (nMenuId)
    {
	case STANDARD_POPUP_MENU:
	    switch(nId)
	    {
		case ID_POPUP_INSERT:
		    AddNewNode(m_CurrentSelection);
		    break;
		case ID_POPUP_DELETE:
		    DeleteMarkedItems();
		    break;
		case ID_POPUP_COPY:
		    CopyMarkedItems();
		    break;
		case ID_POPUP_CUT:
		    CutMarkedItems();
		    break;
		case ID_POPUP_PASTE:
		    PasteItems(CurrentSelectionNode());
		    break;
		case ID_POPUP_LINK:
		    LinkItems(CurrentSelectionNode());
		    break;
	    }
	    break;
	case TOP_POPUP_MENU:
	    switch(nId)
	    {
		case ID_POPUP_INSERT:
		    AddNewNode(Tree());
		    break;
		case ID_POPUP_PASTE:
		    PasteItems(CurrentSelectionNode());
		    break;
		case ID_POPUP_LINK:
		    LinkItems(CurrentSelectionNode());
		    break;
	    }
	    break;
    }
    
}

void CHierarchicalTreeBox::DeleteMarkedItems()
{
    if (m_MultiSelection.card() > 0)
    {
	CSimpleList<CGraphicalTreeNode *> Nodes;

	Nodes = m_MultiSelection;

	CSimpleIterator<CGraphicalTreeNode *> NodeIt;
	NodeIt << &m_MultiSelection;

	while (CGraphicalTreeNode *pNode = NodeIt++)
	{
	    DeleteNode(pNode);
	}
    }
    else
    {
	DeleteCurrentSelection();
    }
}

void CHierarchicalTreeBox::CopyMarkedItems()
{
    EmptyCopyBuffer();
    EmptyCutBuffer();

    CSimpleIterator<CGraphicalTreeNode *> NodeIt;

    NodeIt << &m_MultiSelection;

    while (CGraphicalTreeNode *pNode = NodeIt++)
    {
	m_CopyList += pNode;
    }
}

void CHierarchicalTreeBox::CutMarkedItems()
{
    EmptyCopyBuffer();
    EmptyCutBuffer();

    CSimpleIterator<CGraphicalTreeNode *> NodeIt;

    NodeIt << &m_MultiSelection;

    while (CGraphicalTreeNode *pNode = NodeIt++)
    {
	m_CutList += pNode;
	EnterCutMode(pNode, true);
    }   
}

void CHierarchicalTreeBox::EnterCutMode(CGraphicalTreeNode *pNode, bool bEnterCutMode)
{
    Widget wdgButton = pNode->Item()->mylabel;

    XtSetSensitive(wdgButton, !bEnterCutMode);
}

void CHierarchicalTreeBox::PasteItems(CGraphicalTreeNode *pParentNode)
{
    bool bMovePointer = false;

    CSimpleIterator<CGraphicalTreeNode *> NodeIt;

    if (m_CutList.card() > 0)
    {
	NodeIt << &m_CutList;

	ASSERT(m_CopyList.card() == 0);
	
	bMovePointer = true;

	while (CGraphicalTreeNode *pNode = NodeIt++)
	{
	    graphical_list_item *pItem = pNode->Item();
	    pItem->RemoveFromList();

	    if (pParentNode->Adopt(pNode))
	    {
		InsertNodes(pNode);
		OnItemAdopted(pNode);
	    }
	}
    }
    else
    {
	NodeIt << &m_CopyList;

	while (CGraphicalTreeNode *pNode = NodeIt++)
	{
	    CGraphicalTreeNode *pNewNode = (CGraphicalTreeNode *)pParentNode->Copy(pNode);

	    ASSERT(pNewNode != NULL);

	    InsertNodes(pNewNode);

	    NewItemAdded(pNewNode);
	}
    }

    if (bMovePointer)
    {
	ASSERT(m_CutList.card() > 0);
	NodeIt.reset();

	while (CGraphicalTreeNode *pNode = NodeIt++)
	{
	    m_CopyList += pNode;
	}

	m_CutList.reset();
    }
}

void CHierarchicalTreeBox::InsertNodes(CGraphicalTreeNode *pNode)
{
    InsertNode(pNode);

    int nIndex = 0;

    while (CGraphicalTreeNode *pChild = pNode->Child(nIndex++))
    {
	InsertNodes(pChild);
    }
}

void CHierarchicalTreeBox::OnItemAdopted(CGraphicalTreeNode *pNode)
{
    
}

void CHierarchicalTreeBox::EmptyCopyBuffer()
{
    m_CopyList.reset();
}

void CHierarchicalTreeBox::EmptyCutBuffer()
{
    CSimpleIterator<CGraphicalTreeNode *> NodeIt;

    NodeIt << &m_CutList;
    while (CGraphicalTreeNode *pNode = NodeIt++)
    {
	EnterCutMode(pNode, false);
    }

    m_CutList.reset();
}

void CHierarchicalTreeBox::ReleaseMultiSelection()
{
    CSimpleList<CGraphicalTreeNode *> Nodes;

    Nodes = m_MultiSelection;
    m_MultiSelection.reset();

    for (int nIndex = 0; nIndex < Nodes.card(); nIndex++)
    {
	ColorNode(Nodes[nIndex]);
    }
}

void CHierarchicalTreeBox::AddToMultiSelection(CGraphicalTreeNode *pNode)
{
    if (!InMultiSelection(pNode) && (pNode != NULL) && (Style()&HIERARCHICAL_MULTI_SELECT))
    {
	m_MultiSelection += pNode;
	ColorNode(pNode);
    }
}

void CHierarchicalTreeBox::RemoveFromMultiSelection(CGraphicalTreeNode *pNode)
{
    if (pNode != NULL)
    {
	m_MultiSelection -= pNode;
	ColorNode(pNode);
    }
}

bool CHierarchicalTreeBox::InMultiSelection(CGraphicalTreeNode *pNode)
{
    return (pNode != NULL) && m_MultiSelection.in(pNode);
}

void CHierarchicalTreeBox::ColorNode(CGraphicalTreeNode *pNode)
{
    if (pNode != NULL)
    {
	Widget wdgButton = pNode->Item()->mylabel;
    
	Arg args[5];
	Cardinal argcount=0;
	color_coding color(XtDisplay(wdgButton));
    
	long lColor = 0;

	if (CurrentSelectionNode() == pNode)
	{
	    lColor = color.get_rgb(0.2,0.2,1.);
	}
	else if (m_MultiSelection.in(pNode))
	{
	    lColor = color.get_rgb(0.6,0.6,1.);
	}
	else
	{
	    lColor = BackgroundColor();
	}
    
	XtSetArg(args[argcount],XmNbackground,lColor); argcount++;
	XtSetValues(wdgButton,args,argcount);
    }
}

void CHierarchicalTreeBox::OnOpenNode(graphical_list_item *pItem)
{
    CGraphicalTreeNode *pNode = (CGraphicalTreeNode *)pItem->Data();

    if (pNode != NULL)
    {
	OnOpenNode(pNode);
    }
}

void CHierarchicalTreeBox::OnItemChecked(graphical_list_item *pItem, bool bSet)
{
    CGraphicalTreeNode *pNode = (CGraphicalTreeNode *)pItem->Data();
		
    if (pNode != NULL)
    {
	pNode->Check(bSet);

	SendMessage(EVENT_MASK_CHECK_CHANGED, (void *)pNode->Node().Id());
    }
}

void CHierarchicalTreeBox::GetCheckedItems(CSimpleObjectList<CIdString> &arrStrings)
{
    if (Style() & HIERARCHICAL_CHECK_BOX)
    {
	arrStrings.reset();

	CGraphicalTreeNode *pNode = Tree();

	RecursivelyGetCheckedItems(pNode, arrStrings);
    }
}

void CHierarchicalTreeBox::RecursivelyGetCheckedItems(CGraphicalTreeNode *pNode, CSimpleObjectList<CIdString> &arrStrings)
{
    if (pNode != NULL)
    {
	if (pNode->Checked())
	{
	    arrStrings += pNode->Node();
	}

	int nIndex = 0;

	while (CGraphicalTreeNode *pChild = pNode->Child(nIndex++))
	{
	    RecursivelyGetCheckedItems(pChild, arrStrings);
	}
    }
}

void CHierarchicalTreeBox::CheckItems(CSimpleList<IDENT> &arrIds)
{
    if (Style() & HIERARCHICAL_CHECK_BOX)
    {
	CGraphicalTreeNode *pNode = Tree();

	RecursivelyCheckItems(pNode, arrIds);
    }
}

void CHierarchicalTreeBox::RecursivelyCheckItems(CGraphicalTreeNode *pNode, CSimpleList<IDENT> &arrIds)
{
    if (pNode != NULL)
    {
	bool bSet = (bool)arrIds.in(pNode->Node().Id());

	pNode->Check(bSet);

	/*graphical_list_item *pItem = pNode->Item();

	if (pItem != NULL)
	{
	    pItem->Check(bSet);
	    }*/

	int nIndex = 0;

	while (CGraphicalTreeNode *pChild = pNode->Child(nIndex++))
	{
	    RecursivelyCheckItems(pChild, arrIds);
	}
    }
}

void CHierarchicalTreeBox::Insert(CGraphicalTreeNode *pNode, CTreeNode<CIdString> *pTree)
{
    CGraphicalTreeNode *pNewNode = new CGraphicalTreeNode(*pTree);

    CGraphicalTreeNode *pChild = NULL;

    while (pChild = pNewNode->Child(0))
    {
	pNode->Adopt(pChild);
    }

    RecursivelyAddNodes(pNode, pNode->Item());
}
