/*******************************************************************************
 *
 * libgui Version 0.0.1
 *
 * libgui is a simple framework of GUI classes for X11/Motif widgets written in C++.
 *
 * The full text of the GNU General Public License can be found in the doc 
 * directory.
 *
 * Copyright (C) 2002, 2003 Stefan Hoefer
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * You can contact Stefan Hoefer via email at stefan@hoefer.ch
 *
 ******************************************************************************/

#ifndef HIERARCHICALTREELIST_H

#define HIERARCHICALTREELIST_H

#include "graphicallist.h"
#include <basic/ctree.h>
#include <basic/cstrings.h>

class CGraphicalTreeNode : public CTreeNode<CIdString>
{
 private:
    graphical_list_item *m_pItem;
    unsigned char m_bHideSub:1;
    //unsigned char m_bSpecialNode:1;
    bool m_bChecked;
 private:
    void CommonConstruct();
 public:
    inline graphical_list_item *Item() { return m_pItem; }
    inline void SetItem(graphical_list_item *pItem) { m_pItem = pItem; }
    void operator=(CTreeNode<CIdString> &Tree);
    inline bool HideSub() { return m_bHideSub&0x1;}
    inline void SetHideSub(bool bHide) { if ( bHide ) { m_bHideSub |= 0x1; } else { m_bHideSub &= 0x0; } }
		
    CGraphicalTreeNode(CTreeNode<CIdString> &Tree);
    CGraphicalTreeNode(CGraphicalTreeNode &Tree);
    CGraphicalTreeNode();
		
    void GetHideFlag();
    void SetHideFlag();
		
    //inline void SetSpecialFlag(bool bSet = true) { if ( bSet ) m_bSpecialNode |= 0x1; else m_bSpecialNode &= 0x0; }
    //inline bool SpecialFlag() { return m_bSpecialNode&0x1 ? true : false; }
		
    inline IDENT Id() { return Node().Id(); }
    inline IDENT Type() { return Node().Type(); }
    CGraphicalTreeNode *GetById(IDENT idNumber, IDENT idType=0);
		
    inline CGraphicalTreeNode *Child(int nIndex) { return (CGraphicalTreeNode *)CTreeNode<CIdString>::Child(nIndex); }
    inline CGraphicalTreeNode *Parent() { return (CGraphicalTreeNode *)CTreeNode<CIdString>::Parent(); }
		
    CGraphicalTreeNode *Succ();
    CGraphicalTreeNode *Pred();

    //bool Copy(CGraphicalTreeNode *pNode);
    //CGraphicalTreeNode *Adopt(CGraphicalTreeNode *pNode);

    void Check(bool bCheck = true);
    inline bool Checked() { return m_bChecked; }
};

/*!
*
* \class CHierarchicalTreeBox
*
* \author STH
* \date 15.4.00
*
*/

class CHierarchicalTreeBox : public graphical_list_box
{
 private:
    enum STATES {STATE_IDLE, STATE_INPUT, STATE_BUILDING};
		
    CGraphicalTreeNode *m_Tree;
    static void SelectCallback(Widget w, XtPointer client_data, XmPushButtonCallbackStruct *callback_data);
    static void NewCallback(Widget w, XtPointer client_data, XmAnyCallbackStruct *callback_data);
    static void ValidateInput(Widget w, XtPointer client_data, XmAnyCallbackStruct *callback_data);
    static void LinesCallback(Widget w, XtPointer client_data, XmAnyCallbackStruct *callback_data);
    /*static void DAInput(Widget w, XtPointer client_data, XmDrawingAreaCallbackStruct *callback_data);*/
    static void FocusHandler(Widget w, XtPointer client_data, XEvent *call_data);
				
    void StartInput();
    void EndInput();
    void CancelInput();
		
    void OpenNode(CGraphicalTreeNode *pNode);
    void ComputeCoordinates(Widget Current, Widget Frame, Dimension &x, Dimension &y);
    bool Visible(CGraphicalTreeNode *pNode);
    void CenterNode(CGraphicalTreeNode *pNode);
    void InsertNode(CGraphicalTreeNode *pNode);
    void InsertNodes(CGraphicalTreeNode *pNode);
 private:		
    CGraphicalTreeNode *m_CurrentSelection;
    graphical_list_item *m_wdgCurrentNewButton;
		
    //long m_Style;
    int m_nCurrentState;
		
    Widget m_TextInput;
    int m_nMinWidth;
    GC m_GC;
    Display *m_pDisplay;
		
    Time m_time;
    Widget m_wdgDoubleClick;
    CSimpleList<CGraphicalTreeNode *> m_MultiSelection;
    CSimpleList<CGraphicalTreeNode *> m_CutList;
    CSimpleList<CGraphicalTreeNode *> m_CopyList;
 private:
    //inline CGraphicalTreeNode *CurrentSelectionNode() { return m_CurrentSelection; }
    inline void SetCurrentSelectionNode( CGraphicalTreeNode *pNode) { m_CurrentSelection = pNode; AddToMultiSelection(pNode); }
    unsigned long BackgroundColor(); 	
    void GetChildPositions(graphical_list_item *pItem, int *pChildren);
    void DrawLines(graphical_list_item *pItem);
    void DrawParentLines(graphical_list_item *pItem);
    inline Display *Disp() { return m_pDisplay; }
    void AddNewNode(CGraphicalTreeNode *pNode);
    void DeleteNode(CGraphicalTreeNode *pNode);
    void ReleaseMultiSelection();
    void AddToMultiSelection(CGraphicalTreeNode *pNode);
    void RemoveFromMultiSelection(CGraphicalTreeNode *pNode);
    bool InMultiSelection(CGraphicalTreeNode *pNode);
    void ColorNode(CGraphicalTreeNode *pNode);
    void CopyMarkedItems();
    void CutMarkedItems();
    void EnterCutMode(CGraphicalTreeNode *pNode, bool bEnterCutMode);
    void PasteItems(CGraphicalTreeNode *pNode);
    void LinkItems(CGraphicalTreeNode *pNode);
    void EmptyCopyBuffer();
    void EmptyCutBuffer();
    void DeleteMarkedItems();
    void DeselectAll();
    void RecursivelyGetCheckedItems(CGraphicalTreeNode *pNode, CSimpleObjectList<CIdString> &arrStrings);
    void RecursivelyCheckItems(CGraphicalTreeNode *pNode, CSimpleList<IDENT> &arrIds);
 protected:
    void RecursivelyAddNodes(CGraphicalTreeNode *pNode, graphical_list_item *pItem = NULL);
    void AddNewItemButton(graphical_list_item *pItem);

    virtual void SelectionChanged();
    virtual void TextChanged();
    virtual void NewItemAdded(CGraphicalTreeNode *pNode);
    virtual void ItemDeleted(const CIdString& strItem);

    virtual bool NotifyScroll();
    virtual bool NotifyInput(XmDrawingAreaCallbackStruct *call_data);
    virtual void OnCancel();
    virtual void OnOk();
    virtual void OnTab();
    //virtual void HandleSpecialInput(CGraphicalTreeNode *pNode);
    virtual void InternalCreate(unsigned long lStyle);

    inline graphical_list_item *add_item(const char *cText, int nPosition=-1) { graphical_list_box::add_item(cText,nPosition); }

    virtual bool HasContextMenu();
    virtual void CreateContextMenu(CSimpleObjectList<CIdString> &Items);
    virtual void CreateTopContextMenu(CSimpleObjectList<CIdString> &Items);
    //virtual void OnMapPopup(void *pLauncher);
    virtual void OnPreparePopup(Widget wdgLauncher, bool bShift, CPopupMenuHandle *pHandle);
    virtual void OnPopupMenu(int nMenuId, int nId);
    virtual void OnItemAdopted(CGraphicalTreeNode *pNode);
    virtual void OnOpenNode(CGraphicalTreeNode *pNode);

    void CreateInsertContextMenu(CSimpleObjectList<CIdString> &Items, bool bTop);
    void CreateEditContextMenu(CSimpleObjectList<CIdString> &Items, bool bTop);
    virtual void OnItemChecked(graphical_list_item *pItem, bool bSet);
    virtual void OnOpenNode(graphical_list_item *pItem);
    virtual void OnLosingFocus();
 public:	
    enum { ID_POPUP_INSERT = -1,
	   ID_POPUP_DELETE = -2,
	   ID_POPUP_COPY = -3,
	   ID_POPUP_CUT = -4,
	   ID_POPUP_PASTE = -5,
	   ID_POPUP_LINK = -6};

    inline void DeleteAllItems() { graphical_list_box::DeleteAllItems(); }

    CHierarchicalTreeBox(Widget Parent, unsigned long Style = 0L, int nId=0);
    virtual ~CHierarchicalTreeBox();
				
    void Create(CTreeNode<CIdString> *pTree);
		
    inline CGraphicalTreeNode *Tree() { return m_Tree; }
    inline Widget Container() { return graphical_list_box::Container(); }
    void SetContents(CTreeNode<CIdString> *pTree);
    virtual void SetContents(CGraphicalTreeNode *pTree);
		
    void SetCurrentSelection(CIdString &strData);
    void DeleteCurrentSelection();
    CIdString CurrentSelection();
    inline CGraphicalTreeNode *CurrentSelectionNode() {return m_CurrentSelection; }
    inline void SetMinWidth(int nWidth) { m_nMinWidth = nWidth; }
    inline int MinWidth() { return m_nMinWidth; }
    //inline long Style() { return m_Style; }
		
    void Select(IDENT idNumber, IDENT idType=0);
    void Select(CGraphicalTreeNode *pNode, bool bSelect = true, bool bShift = false, bool bNotifyUser=true);

    inline void Manage() { graphical_list_box::Manage(); }
    inline void Unmanage() { graphical_list_box::Unmanage(); }

    void GetCheckedItems(CSimpleObjectList<CIdString> &arrStrings);
    void CheckItems(CSimpleList<IDENT> &arrIds);

    void Insert(CGraphicalTreeNode *pNode, CTreeNode<CIdString> *pTree);
};

#endif


