/*******************************************************************************
 *
 * libgui Version 0.0.1
 *
 * libgui is a simple framework of GUI classes for X11/Motif widgets written in C++.
 *
 * The full text of the GNU General Public License can be found in the doc 
 * directory.
 *
 * Copyright (C) 2002, 2003 Stefan Hoefer
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * You can contact Stefan Hoefer via email at stefan@hoefer.ch
 *
 ******************************************************************************/

#include "iconbutton.h"
#include "coloring.h"

CIconButton::CIconButton(Widget wdgParent, int nId, const char *strFilename)
    : CButton(wdgParent, nId)
{
    m_Pixmap = XmGetPixmap(XtScreen(wdgParent), (char *)strFilename,0xfff,0xfffffff);
}

CIconButton::CIconButton(Widget wdgParent, int nId, unsigned char *strArray, int nWidth, int nHeight) 
    : CButton(wdgParent, nId)
{
    Display *disp = XtDisplay(wdgParent);
    Screen *screen = XtScreen(wdgParent);
    int depth = DefaultDepth(disp, DefaultScreen(disp));
    Window rootwin = RootWindow(disp, DefaultScreen(disp));

    m_Pixmap = XCreatePixmap(disp, rootwin, nWidth, nHeight, depth);
    unsigned char *pointer = strArray;
    GC gc = XtGetGC(wdgParent, 0, NULL);
    color_coding color;

    for (int nY = 0; nY < nHeight; nY++) 
    {
	for (int nX = 0; nX < nWidth; nX++) 
	{
	    unsigned long lColor = color.get_rgb((unsigned int)pointer[0], (unsigned int)pointer[1], (unsigned int)pointer[2]);

	    XSetForeground(disp,gc,lColor);
	    XDrawPoint(disp, m_Pixmap, gc, nX, nY);

	    pointer += 3;
	    
	}
    }
}

void CIconButton::InternalDestroy() 
{
    XFreePixmap(XtDisplay(Parent()), m_Pixmap);
}

void CIconButton::InternalCreate(unsigned long lStyle)
{
    CButton::InternalCreate(lStyle);

    Cardinal argcount = 0;
    Arg args[20];

    argcount = 0;
    XtSetArg(args[argcount], XmNlabelType, XmPIXMAP); argcount++;
    XtSetArg(args[argcount], XmNlabelPixmap, m_Pixmap); argcount++;
    XtSetArg(args[argcount], XmNwidth, 34); argcount++;
    XtSetArg(args[argcount], XmNheight, 34); argcount++;
    XtSetArg(args[argcount], XmNborderWidth, 0); argcount++;

    XtSetValues(Container(), args, argcount);
}






