/*******************************************************************************
 *
 * libgui Version 0.0.1
 *
 * libgui is a simple framework of GUI classes for X11/Motif widgets written in C++.
 *
 * The full text of the GNU General Public License can be found in the doc 
 * directory.
 *
 * Copyright (C) 2002, 2003 Stefan Hoefer
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * You can contact Stefan Hoefer via email at stefan@hoefer.ch
 *
 ******************************************************************************/

#include "mainframe.h"
#include <basic/simpleiterator.h>
#include <Xm/Notebook.h>
#include "view.h"
#include <basic/globalstring.h>
#include "toolbar.h"
#include <Xm/Form.h>
#include <sys/types.h>
#include <unistd.h>
#include <signal.h>
// #include "load.h"
// #include "save.h"
// #include "new.h"
#include <Xm/Text.h>

#ifdef _DEBUG
static void Dummy(Widget w, XEvent *event, String *params, Cardinal *num_params)
{
    puts("Dummy action activated.");
}
#endif // #ifdef _DEBUG

void CMainFrame::OnQuit()
{
    EndAppLoop();
}

void CMainFrame::Create()
{
#ifdef _DEBUG	
    static XtActionsRec actions[] = {{"Dummy",Dummy}};
    XtAppAddActions(ApplicationContext(),actions,XtNumber(actions));
#endif // #ifdef _DEBUG

    if (m_pPropertiesFile == NULL)
    {
	m_pPropertiesFile = new CPropertiesFile(PropertiesName());
    }

    CGuiElement::Create(get_shell());
	
    CSimpleIterator<CView *> ViewIt;
	
    ViewIt << &m_arrView;

    Arg args[20];
    int nZaehler = 1;
	
    while ( CView *pView = ViewIt++ )
    {	
	pView->Create(m_wdgNotebook);

	Cardinal argcount = 0;
	char strName[256];
	strcpy(strName,pView->Caption());
		
	XmString string = XmStringCreate(strName,XmFONTLIST_DEFAULT_TAG);
		
	XtSetArg(args[argcount],XmNnotebookChildType,XmMAJOR_TAB); argcount++;
	XtSetArg(args[argcount],XmNpageNumber,nZaehler); argcount++;
	XtSetArg(args[argcount],XmNlabelString,string); argcount++;
		
	Widget tab = XmCreatePushButton(m_wdgNotebook,"page",args,argcount);
	XtManageChild(tab);
		
	XmStringFree(string);
	
	nZaehler++;
    }
    
    CStrings strCaption = Title();
    strCaption = strCaption+"@"+getenv("HOSTNAME")+" v"+Version();
    SetCaption(strCaption);
}

void CMainFrame::InternalCreate(unsigned long lStyle)
{
    Arg args[20];
    Cardinal argcount = 0;

    Widget wdgMainWindow = XmCreateMainWindow(get_shell(),"mainwindow",args,argcount);
    XtManageChild(wdgMainWindow);

    //---------------------------------------------------------------------------
    // Prepare the menu
    //---------------------------------------------------------------------------

    argcount = 0;

    Widget menu_bar = XmCreateMenuBar(wdgMainWindow, "MenuBar", args,argcount);
    XtManageChild(menu_bar);

    InternalCreateMenu(menu_bar);
	
    argcount = 0;
    Widget wdgTopForm = XmCreateForm(wdgMainWindow, "topform", args, argcount); 
    XtManageChild(wdgTopForm);

    argcount = 0;
    XtSetArg(args[argcount], XmNleftAttachment, XmATTACH_FORM); argcount++;
    XtSetArg(args[argcount], XmNbottomAttachment, XmATTACH_FORM); argcount++;
    XtSetArg(args[argcount], XmNrightAttachment, XmATTACH_FORM); argcount++;
 	
    if (lStyle&MF_STYLE_TOOLBAR)
    {
	{
	    Arg args2[20];
	    Cardinal argcount2=0;

	    m_pToolBar = new CToolBar(wdgTopForm);
	    m_pToolBar->Create();
	    CreateToolBar(m_pToolBar);
	    ListenTo(m_pToolBar);

	    XtSetArg(args2[argcount2], XmNtopAttachment, XmATTACH_FORM); argcount2++;
	    XtSetArg(args2[argcount2], XmNleftAttachment, XmATTACH_FORM); argcount2++;
	    XtSetArg(args2[argcount2], XmNrightAttachment, XmATTACH_FORM); argcount2++;

	    XtSetValues(m_pToolBar->Container(), args2, argcount2);
	}

	XtSetArg(args[argcount], XmNtopAttachment, XmATTACH_WIDGET); argcount++;
	XtSetArg(args[argcount], XmNtopWidget, m_pToolBar->Container()); argcount++;
    }
    else
    {
	XtSetArg(args[argcount], XmNtopAttachment, XmATTACH_FORM); argcount++;
    }

    XtSetArg(args[argcount],XmNbindingType,XmNONE); argcount++;
    XtSetArg(args[argcount],XmNbackPageSize,0); argcount++;
    XtSetArg(args[argcount],XmNorientation,XmVERTICAL); argcount++;
    XtSetArg(args[argcount],XmNbackPagePlacement,XmTOP_RIGHT); argcount++;
   
    m_wdgNotebook = XmCreateNotebook(wdgTopForm,"notebook",args,argcount);
    XtManageChild(m_wdgNotebook);

    CStrings strX = Properties()->GetEntry("Geometry-X");
    CStrings strY = Properties()->GetEntry("Geometry-Y");
    CStrings strWidth = Properties()->GetEntry("Geometry-Width");
    CStrings strHeight = Properties()->GetEntry("Geometry-Height");
  	
    int nX = 100;
    int nY = 100;
    int nWidth = 400;
    int nHeight = 400;
	  	
    if (strX != "")
    {
	nX = atoi((const char *)strX);
    }
    if (strY != "")
    {
	nY = atoi((const char *)strY);
    }
    if (strWidth != "")
    {
	nWidth = atoi((const char *)strWidth);
    }
    if (strHeight != "")
    {
	nHeight = atoi((const char *)strHeight);
    }
  	
    SetDimensions(nX, nY, nWidth, nHeight);
}

void CMainFrame::QuitHandler(Widget w, XtPointer client_data, XmAnyCallbackStruct *call_data)
{
    CMainFrame *pMain = (CMainFrame *)client_data;
	
    pMain->OnQuit();
	
//      pMain->EndAppLoop();
}

CMainFrame::~CMainFrame()
{

}

Widget CMainFrame::Container() const
{
    return Parent();
}

void CMainFrame::operator+=(CView *pView)
{
    m_arrView += pView;
    pView->m_pMainFrame = this;
}

void CMainFrame::DoModal()
{
    xapplication::wait_for_event();

    Destroy();
}

void CMainFrame::UpdateView(int nId)
{
    CSimpleIterator<CView *> ViewIt;
	
    ViewIt << &m_arrView;
	
    while ( CView *pView = ViewIt++ )
    {
	if ( pView->Id() == nId )
	{
	    pView->Update();
	    return;
	}
    }
}

CView *CMainFrame::View(int nId)
{
    CSimpleIterator<CView *> ViewIt;
	
    ViewIt << &m_arrView;
	
    while ( CView *pView = ViewIt++ )
    {
	if ( pView->Id() == nId )
	{
	    return pView;
	}
    }
    
    return NULL;
}

void CMainFrame::UpdateAllViews(CView *pViewSender)
{
    CSimpleIterator<CView *> ViewIt;
	
    ViewIt << &m_arrView;
	
    while ( CView *pView = ViewIt++ )
    {
	if ( pViewSender != pView )
	{
	    pView->Update();
	}
    }
}

void CMainFrame::CreateHelpMenu(Widget menu_bar)
{
    Widget helpmenu = XmCreatePulldownMenu(menu_bar,"helpmenu",NULL,0);
    Widget help_button = XtVaCreateManagedWidget((char *)LANGUAGE_STRING(IDS_HELP), 
						 xmCascadeButtonWidgetClass, menu_bar, 
						 XmNsubMenuId, helpmenu,
						 NULL);
    XtVaSetValues(menu_bar,XmNmenuHelpWidget, help_button,
		  NULL);
    Widget wdgAbout = XmCreatePushButton(helpmenu, "About", NULL, 0);
    XtAddCallback(wdgAbout, XmNactivateCallback, (XtCallbackProc)AboutHandler, (XtPointer)this);
    XtManageChild(wdgAbout);
}

void CMainFrame::AboutPicture(Widget wdgArea)
{

}

CMainFrame::CAboutDialog::CAboutDialog(CMainFrame *pMainFrame)
    : CModalDialog(DLG_STYLE_OK|DLG_STYLE_SEPARATOR)
{
    m_pMainFrame = pMainFrame;
}

void CMainFrame::CAboutDialog::ExposeHandler(Widget w, XtPointer client_data, XmDrawingAreaCallbackStruct *call_data)
{
    CAboutDialog *dialog = (CAboutDialog *)client_data;

    dialog->m_pMainFrame->AboutPicture(w);
}

void CMainFrame::CAboutDialog::InternalCreate(unsigned long lStyle) 
{
    CModalDialog::InternalCreate(lStyle);

    SetCaption("About");

    Arg args[20];
    Cardinal argcount = 0;

    XtSetArg(args[argcount], XmNleftAttachment, XmATTACH_FORM); argcount++;
    XtSetArg(args[argcount], XmNrightAttachment, XmATTACH_FORM); argcount++;
    XtSetArg(args[argcount], XmNtopAttachment, XmATTACH_FORM); argcount++;
    XtSetArg(args[argcount], XmNbottomAttachment, XmATTACH_WIDGET); argcount++;
    XtSetArg(args[argcount], XmNbottomWidget, Separator()); argcount++;

    Widget wdgTopForm = XmCreateForm(Frame(), "topform", args, argcount);
    XtManageChild(wdgTopForm);
    
    argcount = 0;
    XtSetArg(args[argcount], XmNleftAttachment, XmATTACH_FORM); argcount++;
    XtSetArg(args[argcount], XmNbottomAttachment, XmATTACH_FORM); argcount++;
    XtSetArg(args[argcount], XmNtopAttachment, XmATTACH_FORM); argcount++;
    XtSetArg(args[argcount], XmNwidth, 100); argcount++;
    XtSetArg(args[argcount], XmNheight, 100); argcount++;

    Widget wdgLeftArea = XmCreateDrawingArea(wdgTopForm, "drawingarea", args, argcount);
    XtManageChild(wdgLeftArea);
    XtAddCallback(wdgLeftArea, XmNexposeCallback, (XtCallbackProc)ExposeHandler, (XtPointer)this);

    argcount = 0;
    XtSetArg(args[argcount], XmNleftAttachment, XmATTACH_WIDGET); argcount++;
    XtSetArg(args[argcount], XmNleftWidget, wdgLeftArea); argcount++;
    XtSetArg(args[argcount], XmNbottomAttachment, XmATTACH_FORM); argcount++;
    XtSetArg(args[argcount], XmNtopAttachment, XmATTACH_FORM); argcount++;
    XtSetArg(args[argcount], XmNrightAttachment, XmATTACH_FORM); argcount++;
    XtSetArg(args[argcount], XmNwidth, 200); argcount++;
    XtSetArg(args[argcount], XmNeditable, False); argcount++;
    XtSetArg(args[argcount], XmNcursorPositionVisible, False); argcount++;

    Widget wdgText = XmCreateText(wdgTopForm, "text", args, argcount);
    XtManageChild(wdgText);

    CStrings strText;
    strText = m_pMainFrame->Title();
    strText += "\n";
    strText += "Version ";
    strText += m_pMainFrame->Version();
    strText += "\n";
    strText += " 2002 Stefan Hoefer\nE-Mail <stefan@hoefer.ch>";

    XmTextSetString(wdgText, (char *)(const char *)strText);
}

void CMainFrame::AboutHandler(Widget w, XtPointer client_data, XmAnyCallbackStruct *call_data)
{
    CMainFrame *pFrame = (CMainFrame *)client_data;

    CAboutDialog *dialog = new CAboutDialog(pFrame);
    dialog->Create();
    dialog->DoModal();
    
    delete dialog;
}

void CMainFrame::InternalCreateMenu(Widget menu_bar)
{
    Cardinal argcount = 0;
    Arg args[5];
	
    CreateHelpMenu(menu_bar);

    // Create the file menu
	
    argcount = 0;
    Widget filemenu = XmCreatePulldownMenu(menu_bar,"filemenu",args,argcount);
    Widget filebutton = XtVaCreateManagedWidget((char *)LANGUAGE_STRING(IDS_FILE), 
						xmCascadeButtonWidgetClass, menu_bar, 
						XmNsubMenuId, filemenu,
						NULL);
    Widget exitbutton = XmCreatePushButton(filemenu,(char *)LANGUAGE_STRING(IDS_QUIT), 
					   NULL,0);
    XtManageChild(exitbutton);
    XtAddCallback(exitbutton,XmNactivateCallback, (XtCallbackProc)QuitHandler, (XtPointer)this);
}

void CMainFrame::SetCaption(const char *strCaption)
{
    Arg args[5];
    Cardinal argcount = 0;
	
    XtSetArg(args[argcount],XmNtitle, strCaption); argcount++;
	
    XtSetValues(Parent(),args,argcount);
}

CMainFrame::CMainFrame()
{
    m_pPropertiesFile = NULL;
    m_pToolBar = NULL;
    m_nChildProcessId = 0;
}

void CMainFrame::InternalDestroy()
{
    if (m_pPropertiesFile != NULL)
    {
	// Save the geometry of the application window
	
	int nX = 0;
	int nY = 0;
	int nWidth = 0;
	int nHeight = 0;
  	
	Dimensions(nX, nY, nWidth, nHeight);
  	
	CStrings strGeometry;
	strGeometry.Format("%d", nX);  	
	Properties()->SetEntry("Geometry-X",(const char *)strGeometry);
	
	strGeometry.Format("%d", nY);  	
	Properties()->SetEntry("Geometry-Y",(const char *)strGeometry);

	strGeometry.Format("%d", nWidth);  	
	Properties()->SetEntry("Geometry-Width",(const char *)strGeometry);

	strGeometry.Format("%d", nHeight);  	
	Properties()->SetEntry("Geometry-Height",(const char *)strGeometry);

	delete m_pPropertiesFile;
    }

    CSimpleIterator<CView *> ViewIt;
	
    ViewIt << &m_arrView;
	
    while ( CView *pView = ViewIt++ )
    {
	pView->Destroy();
    }

    CGuiElement::InternalDestroy();
}

void CMainFrame::CreateToolBar(CToolBar *pToolBar)
{
//      pToolBar->Add(ID_NEW, "/home/stefan/grafik/raster/icons/small/new.xpm");
//      pToolBar->Add(ID_LOAD, "/home/stefan/grafik/raster/icons/small/load.xpm");
//      pToolBar->Add(ID_SAVE, "/home/stefan/grafik/raster/icons/small/save.xpm");
//     unsigned char *buffer = new unsigned char[newicon.width*newicon.height*newicon.bytes_per_pixel];
//     NEWICON_RUN_LENGTH_DECODE(buffer, newicon.rle_pixel_data, newicon.width*newicon.height, newicon.bytes_per_pixel);
//     pToolBar->Add(ID_NEW, buffer, newicon.width, newicon.height);
//     delete buffer;
//     buffer = new unsigned char[loadicon.width*loadicon.height*loadicon.bytes_per_pixel];
//     NEWICON_RUN_LENGTH_DECODE(buffer, loadicon.rle_pixel_data, loadicon.width*newicon.height, loadicon.bytes_per_pixel);
//     pToolBar->Add(ID_NEW, buffer, loadicon.width, loadicon.height);
//     delete buffer;
//     buffer = new unsigned char[saveicon.width*saveicon.height*saveicon.bytes_per_pixel];
//     NEWICON_RUN_LENGTH_DECODE(buffer, saveicon.rle_pixel_data, saveicon.width*newicon.height, saveicon.bytes_per_pixel);
//     pToolBar->Add(ID_NEW, buffer, saveicon.width, saveicon.height);
//     delete buffer;
}

void CMainFrame::InternalShowStartup() 
{
}

void CMainFrame::ShowStartup() 
{
    pid_t child_pid;
    
    child_pid = fork();

    if (child_pid != 0) 
    {
	m_nChildProcessId = child_pid;
    }
    else
    {
	InternalShowStartup();
	exit(0);
    }
}

void CMainFrame::HideStartup()
{
    if (m_nChildProcessId != 0)
    {
	kill(m_nChildProcessId, SIGTERM);
	m_nChildProcessId = 0;
    }
}

















