/*******************************************************************************
 *
 * libgui Version 0.0.1
 *
 * libgui is a simple framework of GUI classes for X11/Motif widgets written in C++.
 *
 * The full text of the GNU General Public License can be found in the doc 
 * directory.
 *
 * Copyright (C) 2002, 2003 Stefan Hoefer
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * You can contact Stefan Hoefer via email at stefan@hoefer.ch
 *
 ******************************************************************************/

#ifndef MAINFRAME_H

#define MAINFRAME_H

#define MF_STYLE_TOOLBAR 1

#include "xapplication.h"
#include "guielement.h"
#include <basic/simplelist.h>
#include <basic/propertiesfile.h>
#include "modaldialog.h"

class CView;
class CToolBar;

class CMainFrame : public CGuiElement, private xapplication
{
 protected:
    class CAboutDialog: public CModalDialog
	{
	private:
	    CMainFrame *m_pMainFrame;
	private:
	    static void ExposeHandler(Widget w, XtPointer client_data, XmDrawingAreaCallbackStruct *call_data);
	protected:
	    virtual void InternalCreate(unsigned long lStyle);
	public:
	    CAboutDialog(CMainFrame *pMainFrame);
	};

    friend class CAboutDialog;
 private:
    CSimpleList<CView *> m_arrView;
    CPropertiesFile *m_pPropertiesFile;
    Widget m_wdgNotebook;
    CToolBar *m_pToolBar;
    pid_t m_nChildProcessId;
 protected:
    static void QuitHandler(Widget w, XtPointer client_data, XmAnyCallbackStruct *call_data);
    static void AboutHandler(Widget w, XtPointer client_data, XmAnyCallbackStruct *call_data);
    virtual void InternalCreate(unsigned long lStyle=0L);
    virtual void OnQuit();
    virtual void InternalCreateMenu(Widget menu_bar);
    virtual void InternalDestroy();

    virtual const char *Version() const=NULL;
    virtual const char *Title() const=NULL;
    virtual const char *PropertiesName() const=NULL;
    virtual void CreateToolBar(CToolBar *pToolBar);

    inline CToolBar *ToolBar() const { return m_pToolBar; }
    virtual void InternalShowStartup();
    virtual void CreateHelpMenu(Widget wdgMenuBar);
    virtual void AboutPicture(Widget wdgDrawingArea);
 public:
    enum {ID_NEW, ID_LOAD, ID_SAVE};

    CMainFrame();
    ~CMainFrame();
    void Create();
    virtual Widget Container() const;
		
    void DoModal();
    void operator+=(CView *pView);
    void UpdateAllViews(CView *pView=NULL);
    void UpdateView(int nId);
    void ShowStartup();
    void HideStartup();

    CView *View(int nId);

    inline int Depth() { return xapplication::get_depth(); }
    inline XtAppContext ApplicationContext() { return xapplication::get_app_context(); }
    virtual void SetCaption(const char *strCaption);

    inline CPropertiesFile *Properties() const { return m_pPropertiesFile; }
};

#endif









