/*******************************************************************************
 *
 * libgui Version 0.0.1
 *
 * libgui is a simple framework of GUI classes for X11/Motif widgets written in C++.
 *
 * The full text of the GNU General Public License can be found in the doc 
 * directory.
 *
 * Copyright (C) 2002, 2003 Stefan Hoefer
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * You can contact Stefan Hoefer via email at stefan@hoefer.ch
 *
 ******************************************************************************/

#include "messagedialog.h"
#include <Xm/MessageB.h>
#include <basic/globalstring.h>

CMessageDialog::CMessageDialog(MESSAGEDIALOGTYPE nType)
    : CModalDialog()
{
    m_nType = nType;
}

void CMessageDialog::InternalCreate(unsigned long lStyle)
{    
    CModalDialog::InternalCreate(0L);

    Arg args[20];
    Cardinal argcount = 0;

    XtSetArg(args[argcount], XmNbottomAttachment, XmATTACH_FORM); argcount++;
    XtSetArg(args[argcount], XmNtopAttachment, XmATTACH_FORM); argcount++;
    XtSetArg(args[argcount], XmNleftAttachment, XmATTACH_FORM); argcount++;
    XtSetArg(args[argcount], XmNrightAttachment, XmATTACH_FORM); argcount++;

    XmString strYes = XmStringCreate((char *)LANGUAGE_STRING(IDS_YES), XmFONTLIST_DEFAULT_TAG);
    XmString strNo = XmStringCreate((char *)LANGUAGE_STRING(IDS_NO), XmFONTLIST_DEFAULT_TAG);
    XmString strCancel = XmStringCreate((char *)LANGUAGE_STRING(IDS_CANCEL), XmFONTLIST_DEFAULT_TAG);

    XtSetArg(args[argcount], XmNokLabelString, strYes); argcount++;
    XtSetArg(args[argcount], XmNcancelLabelString, strNo); argcount++;
    XtSetArg(args[argcount], XmNhelpLabelString, strCancel); argcount++;
    XtSetArg(args[argcount], XmNwidth, 400); argcount++;
    XtSetArg(args[argcount], XmNheight, 300); argcount++;

    switch(m_nType)
    {
	case QUESTION:
	    XtSetArg(args[argcount], XmNdialogType, XmDIALOG_QUESTION); argcount++;
	    break;
	case ERROR:
	    XtSetArg(args[argcount], XmNdialogType, XmDIALOG_ERROR); argcount++;
	    break;
	case INFORMATION:
	    XtSetArg(args[argcount], XmNdialogType, XmDIALOG_INFORMATION); argcount++;
	    break;
	case WORKING:
	    XtSetArg(args[argcount], XmNdialogType, XmDIALOG_WORKING); argcount++;
	    break;
	case WARNING:
	    XtSetArg(args[argcount], XmNdialogType, XmDIALOG_WARNING); argcount++;
	    break;
	case MESSAGE:
	    XtSetArg(args[argcount], XmNdialogType, XmDIALOG_MESSAGE); argcount++;
	    break;
    }

    m_wdgMessageBox = XmCreateMessageBox(Frame(), "messagebox", args, argcount);
    XtManageChild(m_wdgMessageBox);

    XtAddCallback(m_wdgMessageBox, XmNokCallback, (XtCallbackProc)YesHandler, (XtPointer)this);
    XtAddCallback(m_wdgMessageBox, XmNcancelCallback, (XtCallbackProc)NoHandler, (XtPointer)this);
    XtAddCallback(m_wdgMessageBox, XmNhelpCallback, (XtCallbackProc)CancelHandler, (XtPointer)this);

    XmStringFree(strYes);
    XmStringFree(strNo);
    XmStringFree(strCancel);
}

void CMessageDialog::SetText(const char *strCaption)
{
    Arg args[20];
    Cardinal argcount = 0;

    XmString string = XmStringCreate((char *)strCaption, XmFONTLIST_DEFAULT_TAG);

    XtSetArg(args[argcount], XmNmessageString, string); argcount++;
    XtSetValues(m_wdgMessageBox, args, argcount);
    
    XmStringFree(string);
}

void CMessageDialog::YesHandler(Widget w, XtPointer client_data, XmAnyCallbackStruct *call_data)
{
    CMessageDialog *dialog = (CMessageDialog *)client_data;
    dialog->OnYes();
}

void CMessageDialog::NoHandler(Widget w, XtPointer client_data, XmAnyCallbackStruct *call_data)
{
    CMessageDialog *dialog = (CMessageDialog *)client_data;
    dialog->OnNo();
}

void CMessageDialog::CancelHandler(Widget w, XtPointer client_data, XmAnyCallbackStruct *call_data)
{
    CMessageDialog *dialog = (CMessageDialog *)client_data;
    dialog->OnCancel();
}
