/*******************************************************************************
 *
 * libgui Version 0.0.1
 *
 * libgui is a simple framework of GUI classes for X11/Motif widgets written in C++.
 *
 * The full text of the GNU General Public License can be found in the doc 
 * directory.
 *
 * Copyright (C) 2002, 2003 Stefan Hoefer
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * You can contact Stefan Hoefer via email at stefan@hoefer.ch
 *
 ******************************************************************************/

#include "popupmenuhandle.h"
#include <Xm/Xm.h>
#include <Xm/RowColumn.h>
#include <Xm/PushBG.h>
#include <Xm/Separator.h>
#include <basic/simpleobjectiterator.h>
#include "guimanager.h"
#include <basic/basicstack.h>
#include <Xm/CascadeB.h>
#include <Xm/ToggleBG.h>
#include <Xm/ToggleB.h>

CPopupMenuHandle::CPopupMenuHandle(CGuiManager *pManager, CSimpleObjectList<CIdString> &Items, int nMenuId)
{
    m_pManager = pManager;
    m_wdgPopupMenu = NULL;
    m_nMenuId = nMenuId;
    m_arrItems = Items;
}

/*void CPopupMenuHandle::MapHandler(Widget w, XtPointer client_data, XmRowColumnCallbackStruct *call_data)
{
    //TRACE(XtName(w));
    TRACE(XtName(call_data->widget));

    CPopupMenuHandle *pHandle = (CPopupMenuHandle *)client_data;

    pHandle->m_pManager->OnMapPopup(NULL);
}*/

CPopupMenuContext::CPopupMenuContext(int nType)
{
    m_nType = nType;
}

void CPopupMenuHandle::AddPopup(Widget wdgParent)
{
    if (m_wdgPopupMenu == NULL)
    {
	Arg args[5];
	Cardinal argcount = 0;
      
	XtSetArg(args[argcount], XmNuserData, (XtPointer)this); argcount++;
	XtSetArg(args[argcount], XmNpopupEnabled, XmPOPUP_AUTOMATIC_RECURSIVE); argcount++;
      
	//m_wdgPopupMenu = XmCreatePopupMenu(wdgParent, "popupmenu", args, argcount);
	//m_wdgPopupMenu = XmCreatePopupMenu(m_pManager->Container(), "popupmenu", args, argcount);
	m_wdgPopupMenu = XmCreatePopupMenu(m_pManager->PopupContainer(), "popupmenu", args, argcount);

	if (m_pManager->PopupContainer() != wdgParent)
	{
	    XtAddCallback(m_pManager->PopupContainer(), XmNpopupHandlerCallback, (XtCallbackProc)PopupSelection, (XtPointer)this);
	}

	//XtAddCallback(m_wdgPopupMenu, XmNmapCallback, MapHandler, (XtPointer)this);

	CSimpleObjectIterator<CIdString> m_StringIt;
	m_StringIt << &m_arrItems;
      
	CIdString* pString = NULL;

	basic_stack<Widget> stack;
	stack.push(m_wdgPopupMenu);

	while (pString = m_StringIt++)
	{
	    if (pString->Type() == NORMAL) 
	    {
		if (pString->Id() == 0)
		{
		    argcount = 0;

		    Widget Separator = XmCreateSeparator(m_wdgPopupMenu, "popupmenuseparator", args, argcount);
		    XtManageChild(Separator);
		}
		else
		{
		    argcount = 0;
		    char *strBuffer = new char[pString->Length()+1];
		    strcpy(strBuffer, (const char *)*pString);
		    XmString string = XmStringCreateLocalized(strBuffer);
		    delete strBuffer;
		    XtSetArg(args[argcount], XmNlabelString, string); argcount++;
		    //XtSetArg(args[argcount], XmNuserData, (XtPointer)pString->Id()); argcount++;
		
		    Widget Button = XmCreatePushButtonGadget(stack.top(), "popupmenubutton", args, argcount); 
		    XtManageChild(Button);
		
		    CPopupMenuContext Context(NORMAL);
		    Context.m_nId = pString->Id();
		    //Context.m_nMenuId = MenuId();
		    Context.m_wdgButton = Button;
		    Context.m_pHandle = this;
		    //Context.m_pLauncher = pLauncher;

		    CPopupMenuContext *pContext = m_arrContext.Add(Context);

		    XtAddCallback(Button,XmNactivateCallback,(XtCallbackProc)PopupMenuHandler,(XtPointer)pContext);
		    XmStringFree(string);
		}
	    }
	    else if (pString->Type() == TOGGLE)
	    {
		if (pString->Id() == 0)
		{
		    argcount = 0;

		    Widget Separator = XmCreateSeparator(m_wdgPopupMenu, "popupmenuseparator", args, argcount);
		    XtManageChild(Separator);
		}
		else
		{
		    argcount = 0;
		    char *strBuffer = new char[pString->Length()+1];
		    strcpy(strBuffer, (const char *)*pString);
		    XmString string = XmStringCreateLocalized(strBuffer);
		    delete strBuffer;
		    XtSetArg(args[argcount], XmNlabelString, string); argcount++;
		
		    Widget Button = XmCreateToggleButtonGadget(stack.top(), "popuptogglebutton", args, argcount); 
		    XtManageChild(Button);
		
		    CPopupMenuContext Context(TOGGLE);
		    Context.m_nId = pString->Id();
		    Context.m_wdgButton = Button;
		    Context.m_pHandle = this;

		    CPopupMenuContext *pContext = m_arrContext.Add(Context);

		    XtAddCallback(Button,XmNvalueChangedCallback,(XtCallbackProc)PopupMenuHandler,(XtPointer)pContext);
		    XmStringFree(string);
		}
	    }
	    else if (pString->Type() == RADIOSUBMENU)
	    {
		argcount = 0;
		XtSetArg(args[argcount], XmNradioBehavior, True); argcount++;
		XtSetArg(args[argcount], XmNradioAlwaysOne, True); argcount++;
		Widget submenu = XmCreatePulldownMenu(stack.top(), "submenu", args, argcount);
		//Widget radiobox = XmCreateRadioBox(submenu, "radiobox", args, argcount);
		//XtManageChild(radiobox);

		Widget menubutton = XtVaCreateManagedWidget((const char *)*pString, 
							    xmCascadeButtonWidgetClass, stack.top(), 
							    XmNsubMenuId, submenu,
							    NULL);
		//stack.push(radiobox);
		stack.push(submenu);
	    }
	    else if (pString->Type() == SUBMENU)
	    {
		argcount = 0;
		Widget submenu = XmCreatePulldownMenu(stack.top(),"submenu",args,argcount);
		Widget menubutton = XtVaCreateManagedWidget((const char *)*pString, 
							    xmCascadeButtonWidgetClass, stack.top(), 
							    XmNsubMenuId, submenu,
							    NULL);
		stack.push(submenu);
	    }
	    else if (pString->Type() == SUBMENUEND)
	    {
		stack.pop();
	    }
	}
    }
    //else
    //{
    XmAddToPostFromList(m_wdgPopupMenu, wdgParent);
    //}

    XtAddCallback(wdgParent, XmNpopupHandlerCallback, (XtCallbackProc)PopupSelection, (XtPointer)this);
}

void CPopupMenuHandle::EnableItem(int nId, bool bEnable)
{
    CSimpleObjectIterator<CPopupMenuContext> ContextIt;

    ContextIt << &m_arrContext;

    while (CPopupMenuContext *pContext = ContextIt++)
    {
	if (pContext->Id() == nId)
	{
	    XtSetSensitive(pContext->Button(), bEnable);
	}
    }
}

void CPopupMenuHandle::Select(int nId, bool bSelect)
{
    CSimpleObjectIterator<CPopupMenuContext> ContextIt;

    ContextIt << &m_arrContext;

    while (CPopupMenuContext *pContext = ContextIt++)
    {
	if (pContext->Id() == nId)
	{
	    XmToggleButtonSetState(pContext->Button(), bSelect ? True : False, False);
	}
    }
}

int CPopupMenuContext::MenuId() const
{ 
    return m_pHandle->MenuId(); 
}

CGuiManager *CPopupMenuContext::Manager() const
{ 
    return m_pHandle->Manager(); 
}

void CPopupMenuHandle::PopupSelection(Widget w, XtPointer client_data, XmAnyCallbackStruct *call_data)
{
    CPopupMenuHandle *pMenuHandle = (CPopupMenuHandle *)client_data;

    XEvent *event = call_data->event;

    bool bShift=false;

    if (event->type == ButtonPress)
    {
	XButtonEvent *pButtonEvent = (XButtonEvent *)event;
	
	bShift = pButtonEvent->state&ShiftMask;
    }

    pMenuHandle->m_pManager->OnPreparePopup(w, bShift, pMenuHandle);
}

bool CPopupMenuHandle::operator==(const CPopupMenuHandle& handle) const
{
    return ((m_wdgPopupMenu == handle.m_wdgPopupMenu)
	    && (m_nMenuId == handle.m_nMenuId)
	    && (m_arrItems == handle.m_arrItems));
}

void CPopupMenuHandle::PopupMenuHandler(Widget w, XtPointer client_data, XmPushButtonCallbackStruct *call_data)
{
    CPopupMenuContext *pContext = (CPopupMenuContext *)client_data;

    if (pContext->Type() == NORMAL)
    {
	CGuiManager *pManager = pContext->Manager();

	pManager->OnPopupMenu(pContext->MenuId(), pContext->Id());
    }
    else if (pContext->Type() == TOGGLE)
    {
	XmToggleButtonCallbackStruct *pCallData = (XmToggleButtonCallbackStruct *)call_data;

	CGuiManager *pManager = pContext->Manager();
	
	pManager->OnPopupMenu(pContext->MenuId(), pContext->Id(), pCallData->set);
    }
}











