/*******************************************************************************
 *
 * libgui Version 0.0.1
 *
 * libgui is a simple framework of GUI classes for X11/Motif widgets written in C++.
 *
 * The full text of the GNU General Public License can be found in the doc 
 * directory.
 *
 * Copyright (C) 2002, 2003 Stefan Hoefer
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * You can contact Stefan Hoefer via email at stefan@hoefer.ch
 *
 ******************************************************************************/

#ifndef POPUPMENUHANDLE
#define POPUPMENUHANDLE

#include <Xm/Xm.h>
#include <basic/idstring.h>
#include <basic/simpleobjectlist.h>

//extern void XmAddToPostFromList(Widget wdgMenu, Widget wdgPostFromMenu);

class CGuiManager;
class CPopupMenuHandle;

class CPopupMenuContext
{
    friend class CPopupMenuHandle;
 private:
    CPopupMenuHandle *m_pHandle;
    int m_nId;
    Widget m_wdgButton;
    int m_nType;
 public:
    CPopupMenuContext(int nType);
    CGuiManager *Manager() const;
    inline int Id() const { return m_nId; }
    int MenuId() const;
    inline Widget Button() const { return m_wdgButton; }
    inline CPopupMenuHandle *MenuHandle() const { return m_pHandle; }
    inline int Type() const { return m_nType; }

    bool operator==(const CPopupMenuContext &Context) const { return ((Manager() == Context.Manager()) && (m_nId == Context.m_nId) && (MenuId() == Context.MenuId())); }
};

class CPopupMenuHandle
{
    friend class CGuiManager;
 private:
    CGuiManager *m_pManager;
    Widget m_wdgPopupMenu;
    int m_nMenuId;
    CSimpleObjectList<CIdString> m_arrItems;
    CSimpleObjectList<CPopupMenuContext> m_arrContext;
 private:
    static void PopupMenuHandler(Widget w, XtPointer client_data, XmPushButtonCallbackStruct *call_data);
    //static void MapHandler(Widget w, XtPointer client_data, XmRowColumnCallbackStruct *call_data);
    static void PopupSelection(Widget w, XtPointer client_data, XmAnyCallbackStruct *call_data);
 public:
    enum TYPE {NORMAL=0, SUBMENU, SUBMENUEND, TOGGLE, RADIOSUBMENU};
    
    CPopupMenuHandle(CGuiManager *pManager, CSimpleObjectList<CIdString>& arrItems, int nMenuId);
    inline CGuiManager *Manager() { return m_pManager; }
    void AddPopup(Widget wdgParent);
    inline int MenuId() { return m_nMenuId; }

    bool operator==(const CPopupMenuHandle& handle) const;

    void EnableItem(int nId, bool bEnable);
    void Select(int nId, bool bSelect=true);
};

#endif













