/*******************************************************************************
 *
 * libgui Version 0.0.1
 *
 * libgui is a simple framework of GUI classes for X11/Motif widgets written in C++.
 *
 * The full text of the GNU General Public License can be found in the doc 
 * directory.
 *
 * Copyright (C) 2002, 2003 Stefan Hoefer
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * You can contact Stefan Hoefer via email at stefan@hoefer.ch
 *
 ******************************************************************************/

#include "selectionlist.h"
#include <Xm/List.h>

CSelectionList::CSelectionList(Widget wdgParent, unsigned long lStyle, int nId)
    : CCommonListBox(wdgParent, lStyle, nId)
{
    m_List = NULL;
}

Widget CSelectionList::Container() const
{
    return m_List;
}

Widget CSelectionList::InternalList()
{
    return m_List;
}

void CSelectionList::SelectionChanged()
{
    SendMessage(EVENT_MASK_SELECTION_CHANGED);
}

void CSelectionList::InternalListCreate(unsigned long lStyle)
{
    Arg args[20];
    Cardinal argcount = 0;

    if (lStyle & LB_STYLE_CONSTANTWIDTH)
    {
	XtSetArg(args[argcount], XmNlistSizePolicy, XmCONSTANT); argcount++;
    }

    if (lStyle &  LB_STYLE_MULTISELECT)
    {
	XtSetArg(args[argcount], XmNselectionPolicy, XmMULTIPLE_SELECT); argcount++;
    }
    else
    {
	XtSetArg(args[argcount], XmNselectionPolicy, XmSINGLE_SELECT); argcount++;
    }

    XtSetArg(args[argcount], XmNheight, 200); argcount++;
    XtSetArg(args[argcount], XmNlistSizePolicy, XmCONSTANT); argcount++;

    m_List = XmCreateList(Parent(), "selectionlist", args, argcount);
    XtManageChild(m_List);

    if (!(lStyle & LB_STYLE_MULTISELECT))
    {
	XtAddCallback(m_List, XmNsingleSelectionCallback, (XtCallbackProc)HandleSingleSelection, (XtPointer)this);
    }
}

void CSelectionList::HandleSingleSelection(Widget w, XtPointer client_data, XmAnyCallbackStruct *call_data)
{
    CSelectionList *pList = (CSelectionList *)client_data;
    if (!pList->InternalSelect())
    {
	Arg args[20];
	Cardinal argcount = 0;

	unsigned int *pPositions;
	int nNumberPositions;

	XtSetArg(args[argcount], XmNselectedPositions, &pPositions); argcount++;
	XtSetArg(args[argcount], XmNselectedPositionCount, &nNumberPositions); argcount++;

	XtGetValues(pList->m_List, args, argcount);

	if (nNumberPositions == 1)
	{
	    pList->Select((int)(*pPositions-1));
	
	    //XtFree((char *)pPositions);
	}
	else
	{
	    pList->Select(-1);
	}

	pList->SelectionChanged();
    }
}

void CSelectionList::GetMultipleSelections(CSimpleList<CIdString *> &arrSelections)
{
    ASSERT(arrSelections.card() == 0);

    if (Style() & LB_STYLE_MULTISELECT)
    {
	Arg args[20];
	Cardinal argcount = 0;

	unsigned int *pPositions;
	int nNumberPositions;

	XtSetArg(args[argcount], XmNselectedPositions, &pPositions); argcount++;
	XtSetArg(args[argcount], XmNselectedPositionCount, &nNumberPositions); argcount++;

	XtGetValues(m_List, args, argcount);

	for (int nIndex = 0; nIndex < nNumberPositions; nIndex++)
	{
	    CIdString *pString = Strings()[pPositions[nIndex]-1];

	    ASSERT(pPositions[nIndex] >= 1 && pPositions[nIndex] <= Strings().card());

	    arrSelections += pString;
	}

	//XtFree((char *)pPositions);
    }
}






