/*******************************************************************************
 *
 * libgui Version 0.0.1
 *
 * libgui is a simple framework of GUI classes for X11/Motif widgets written in C++.
 *
 * The full text of the GNU General Public License can be found in the doc 
 * directory.
 *
 * Copyright (C) 2002, 2003 Stefan Hoefer
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * You can contact Stefan Hoefer via email at stefan@hoefer.ch
 *
 ******************************************************************************/

#include "textfield.h"
#include <Xm/TextF.h>
#include <X11/keysym.h>

XtActionsRec CTextField::m_actions[] = {{"TFOnTab",CTextField::HandleTab}};
bool CTextField::m_bActionsAdded = false;

void CTextField::HandleTab(Widget w, XEvent *event, String *params, Cardinal *num_params)
{
    Arg args[5];
    Cardinal argcount = 0;
    XtPointer pointer;
	
    XtSetArg(args[argcount],XmNuserData,&pointer); argcount++;
	
    XtGetValues(w,args,argcount);
	
    ((CTextField *)pointer)->OnTab();
}

CTextField::CTextField(Widget wdgParent)
    : CTextControl(wdgParent)
{

}

CTextField::CTextField()
    : CTextControl()
{

}

void CTextField::InputDone()
{

}

void CTextField::InputCancelled()
{

}

void CTextField::OnTab()
{

}

void CTextField::InternalCreate(unsigned long lStyle)
{
    if ( !m_bActionsAdded )
    {
	XtAppAddActions(ApplicationContext(),m_actions,XtNumber(m_actions));
	m_bActionsAdded = true;
    }

    Arg args[20];
    Cardinal argcount = 0;
	
    XtSetArg(args[argcount],XmNuserData,this); argcount++;
    if ( lStyle&TF_STYLE_READONLY)
    {
	XtSetArg(args[argcount],XmNeditable,False); argcount++;
    }

    SetTextControl(XmCreateTextField(Parent(),"textfield",args,argcount));
    XtManageChild(Container());

    XtAddCallback(Container(),XmNactivateCallback,(XtCallbackProc)HandleInput,(XtPointer)this);
    XtAddCallback(Container(),XmNlosingFocusCallback,(XtCallbackProc)HandleLosingFocus,(XtPointer)this);
    XtAddCallback(Container(),XmNvalueChangedCallback,(XtCallbackProc)HandleChangedValue,(XtPointer)this);

    XtAddEventHandler(Container(),KeyPressMask,FALSE,(void (*)(_WidgetRec *, void *, _XEvent *, char *))KeyboardInputHandler,(XtPointer)this);
		
    //XtTranslations trans = XtParseTranslationTable("c<Key>c: TFCancelInput()");
    if (lStyle & TF_STYLE_ONTAB)
    {
	XtTranslations trans = XtParseTranslationTable("<Key>Tab: TFOnTab()");
		
	XtOverrideTranslations(Container(),trans);
    }
}

void CTextField::KeyboardInputHandler(Widget w, XtPointer client_data, XEvent *call_data)
{
    XKeyEvent *KeyEvent = (XKeyEvent *)call_data;

    CTextField *pTextField = (CTextField *)client_data;

    switch ( XLookupKeysym(KeyEvent,0) )
    {
	/*case XK_Left:
	  puts("left");
	  break;
	  case XK_Right:
	  puts("Right");
	  break;
	  case XK_Up:
	  puts("Up");
	  break;
	  case XK_Down:
	  puts("Down");
	  break;*/
	case XK_Escape:
	    pTextField->InputCancelled();
	    break;
    }
    
}

/*Widget CTextField::Container()
{
	return m_wdgTextField;
}*/

void CTextField::HandleInput(Widget w, XtPointer client_data, XmAnyCallbackStruct *call_data)
{
    CTextField *pText = (CTextField *)client_data;

    pText->InputDone();
}

/*void CTextField::SetText(const char *strText)
{
	char *strBuffer = new char[strlen(strText)+1];
	
	strcpy(strBuffer,strText);
	
	XmTextFieldSetString(m_wdgTextField,strBuffer);	

	delete strBuffer;
	
	m_strText = strText;
}

const char *CTextField::Text()
{
	char *strBuffer = XmTextFieldGetString(m_wdgTextField);
	
	m_strText = strBuffer;
	
	XtFree(strBuffer);
	
	return (const char *)m_strText;
}*/

void CTextField::HandleLosingFocus(Widget w, XtPointer client_data, XmAnyCallbackStruct *call_data)
{
    CTextField *pTextField = (CTextField *)client_data;
	
    pTextField->OnLosingFocus();
}

void CTextField::HandleChangedValue(Widget w, XtPointer client_data, XmAnyCallbackStruct *call_data)
{
    CTextField *pTextField = (CTextField *)client_data;	

    pTextField->SendMessage(EVENT_MASK_VALUE_CHANGED);
}

void CTextField::OnLosingFocus()
{

}

void CTextField::InternalSetText(char *strText)
{
    XmTextFieldSetString(Container(),strText);	
}

char *CTextField::InternalText()
{
    return XmTextFieldGetString(Container());
}

CTextField::~CTextField()
{

}


