/*******************************************************************************
 *
 * libgui Version 0.0.1
 *
 * libgui is a simple framework of GUI classes for X11/Motif widgets written in C++.
 *
 * The full text of the GNU General Public License can be found in the doc 
 * directory.
 *
 * Copyright (C) 2002, 2003 Stefan Hoefer
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * You can contact Stefan Hoefer via email at stefan@hoefer.ch
 *
 ******************************************************************************/

#include "toolbar.h"
#include <Xm/RowColumn.h>
#include "iconbutton.h"
#include <basic/simpleiterator.h>

CToolBar::CToolBar(Widget wdgParent)
    : CGuiElement(wdgParent)
{
    m_wdgContainer = NULL;
}

void CToolBar::InternalCreate(unsigned long lStyle)
{
    Arg args[20];
    Cardinal argcount = 0;

    XtSetArg(args[argcount], XmNorientation, XmHORIZONTAL); argcount++;
    XtSetArg(args[argcount], XmNspacing, 1); argcount++;
    XtSetArg(args[argcount], XmNmarginWidth, 1); argcount++;
    XtSetArg(args[argcount], XmNmarginHeight, 1); argcount++;

    m_wdgContainer = XmCreateRowColumn(Parent(), "toolbarcontainer", args, argcount);
    XtManageChild(m_wdgContainer);
}

Widget CToolBar::Container() const
{
    return m_wdgContainer;
}

void CToolBar::OnMessage(CMessageClient *pClient, unsigned long lMask, void *pData)
{
    int nId = pClient->Id();

    SendMessage(EVENT_MASK_BUTTON_PUSHED, (void *)nId);
}

void CToolBar::Add(int nId, const char *strFileName)
{
    CIconButton *pButton = new CIconButton(m_wdgContainer, nId, strFileName);
    pButton->Create();    
    ListenTo(pButton);

    m_arrButtons += pButton;
}

void CToolBar::Add(int nId, unsigned char *strArray, int nWidth, int nHeight)
{
    CIconButton *pButton = new CIconButton(m_wdgContainer, nId, strArray, nWidth, nHeight);
    pButton->Create();    
    ListenTo(pButton);

    m_arrButtons += pButton;    
}

void CToolBar::InternalDestroy()
{
    CSimpleIterator<CIconButton *> ButtonIt;

    ButtonIt << &m_arrButtons;

    while (CIconButton *pButton = ButtonIt++)
    {
	delete pButton;
    }

    m_arrButtons.reset();
}
